import logging
import os

import argparse
from six.moves import configparser

from nose2 import config, events, util


log = logging.getLogger(__name__)
__unittest = True


class Session(object):

    """Configuration session.

    Encapsulates all configuration for a given test run.

    .. attribute :: argparse

       An instance of :class:`argparse.ArgumentParser`. Plugins can
       use this directly to add arguments and argument groups, but
       *must* do so in their ``__init__`` methods.

    .. attribute :: pluginargs

       The argparse argument group in which plugins (by default) place
       their command-line arguments. Plugins can use this directly to
       add arguments, but *must* do so in their ``__init__`` methods.

    .. attribute :: hooks

       The :class:`nose2.events.PluginInterface` instance contains
       all available plugin methods and hooks.

    .. attribute :: plugins

       The list of loaded -- but not necessarily *active* -- plugins.

    .. attribute :: verbosity

       Current verbosity level. Default: 1.

    .. attribute :: startDir

       Start directory of test run. Test discovery starts
       here. Default: current working directory.

    .. attribute :: topLevelDir

       Top-level directory of test run. This directory is added to
       sys.path. Default: starting directory.

    .. attribute :: libDirs

       Names of code directories, relative to starting
       directory. Default: ['lib', 'src']. These directories are added
       to sys.path and discovery if the exist.

    .. attribute :: testFilePattern

       Pattern used to discover test module files. Default: test*.py

    .. attribute :: testMethodPrefix

       Prefix used to discover test methods and functions: Default: 'test'.

    .. attribute :: unittest

       The config section for nose2 itself.

    """
    configClass = config.Config
    
    def __init__(self):
        self.argparse = argparse.ArgumentParser(prog='nose2', add_help=False)
        self.pluginargs = self.argparse.add_argument_group(
            'plugin arguments',
            'Command-line arguments added by plugins:')
        self.config = configparser.ConfigParser()
        self.hooks = events.PluginInterface()
        self.plugins = []
        self.verbosity = 1
        self.startDir = None
        self.topLevelDir = None
        self.testResult = None
        self.testLoader = None
        self.logLevel = logging.WARN

    def get(self, section):
        """Get a config section.

        :param section: The section name to retreive.
        :returns: instance of self.configClass.

        """
        # FIXME cache these
        items = []
        if self.config.has_section(section):
            items = self.config.items(section)
        return self.configClass(items)

    def loadConfigFiles(self, *filenames):
        """Load config files.

        :param filenames: Names of config files to load.

        Loads all names files that exist into ``self.config``.

        """
        self.config.read(filenames)

    def loadPlugins(self, modules=None, exclude=None):
        """Load plugins.

        :param modules: List of module names from which to load plugins.

        """
        # plugins set directly
        if modules is None:
            modules = []
        if exclude is None:
            exclude = []
        # plugins mentioned in config file(s)
        cfg = self.unittest
        more_plugins = cfg.as_list('plugins', [])
        cfg_exclude = cfg.as_list('exclude-plugins', [])
        exclude.extend(cfg_exclude)
        exclude = set(exclude)
        all_ = (set(modules) | set(more_plugins)) - exclude
        log.debug("Loading plugin modules: %s", all_)
        for module in all_:
            self.loadPluginsFromModule(util.module_from_name(module))
        self.hooks.pluginsLoaded(events.PluginsLoadedEvent(self.plugins))

    def loadPluginsFromModule(self, module):
        """Load plugins from a module.

        :param module: A python module containing zero or more plugin
                       classes.

        """
        avail = []
        for entry in dir(module):
            try:
                item = getattr(module, entry)
            except AttributeError:
                pass
            try:
                if issubclass(item, events.Plugin):
                    avail.append(item)
            except TypeError:
                pass
        for cls in avail:
            log.debug("Plugin is available: %s", cls)
            plugin = cls(session=self)
            self.plugins.append(plugin)
            for method in self.hooks.preRegistrationMethods:
                if hasattr(plugin, method):
                    self.hooks.register(method, plugin)

    def registerPlugin(self, plugin):
        """Register a plugin.

        :param plugin: A `nose2.events.Plugin` instance.

        Register the plugin with all methods it implements.

        """
        log.debug("Register active plugin %s", plugin)
        if plugin not in self.plugins:
            self.plugins.append(plugin)
        for method in self.hooks.methods:
            if hasattr(plugin, method):
                log.debug("Register method %s for plugin %s", method, plugin)
                self.hooks.register(method, plugin)

    def setStartDir(self):
        if self.startDir is None:
            self.startDir = self.unittest.as_str('start-dir', '.')

    def prepareSysPath(self):
        """Add code directories to sys.path"""
        tld = self.topLevelDir
        sd = self.startDir
        if tld is None:
            tld = sd
        tld = os.path.abspath(tld)
        util.ensure_importable(tld)
        for libdir in self.libDirs:
            libdir = os.path.abspath(os.path.join(tld, libdir))
            if os.path.exists(libdir):
                util.ensure_importable(libdir)

    # convenience properties
    @property
    def libDirs(self):
        return self.unittest.as_list('code-directories', ['lib', 'src'])

    @property
    def testFilePattern(self):
        return self.unittest.as_str('test-file-pattern', 'test*.py')

    @property
    def testMethodPrefix(self):
        return self.unittest.as_str('test-method-prefix', 'test')

    @property
    def unittest(self):
        return self.get('unittest')

    def isPluginLoaded(self, pluginName):
        """Returns ``True`` if a given plugin is loaded.

        :param pluginName: the name of the plugin module: e.g. "nose2.plugins.layers".

        """
        for plugin in self.plugins:
            if pluginName == plugin.__class__.__module__:
                return True
        return False



import sys

import six

from nose2.plugins import printhooks
from nose2 import events, session
from nose2.tests._common import TestCase


class CustomEvent(events.Event):

    _attrs = events.Event._attrs + ('args',)

    def __init__(self, args, **kw):
        self.args = args
        super(CustomEvent, self).__init__(**kw)


class TestPluginA(events.Plugin):

    def register(self):
        super(TestPluginA, self).register()
        self.addMethods('pluginHookA')

    def register_with_nested_hook(self):
        super(TestPluginA, self).register()
        self.addMethods('pluginHookB')


class TestPluginB(events.Plugin):

    def pluginHookA(self, event):
        event.handled = True
        return "TestPluginB.pluginHookA"


class TestPluginC(events.Plugin):

    def register(self):
        super(TestPluginC, self).register()
        self.addMethods('pluginHookB1')

    def pluginHookB(self, event):
        nested_event = CustomEvent('level_two_args')
        self.session.hooks.pluginHookB1(nested_event)
        event.handled = True
        return "TestPluginC.pluginHookB"


class IndentAwarePlugin(events.Plugin):

    def pluginHookA(self, event):
        event.handled = True
        return printhooks.INDENT.pop()


class TestPrintHooksPlugin(TestCase):
    tags = ['unit']

    def setUp(self):
        self.err = sys.stderr
        self.buf = six.StringIO()
        sys.stderr = self.buf
        self.addCleanup(self.restore_stderr)

        self.session = session.Session()
        self.print_hooks_plugin = printhooks.PrintHooks(session=self.session)
        self.plugin_a = TestPluginA(session=self.session)
        self.plugin_b = TestPluginB(session=self.session)

    def restore_stderr(self):
        sys.stderr = self.err

    def test_traces_hooks_created_after_own_registration(self):
        self.print_hooks_plugin.register()
        self.plugin_a.register()
        self.plugin_b.register()

        event = CustomEvent('args')
        result = self.session.hooks.pluginHookA(event)
        self.assertEqual(result, "TestPluginB.pluginHookA")
        self.assertEqual("\n"
                         "pluginHookA: "
                         "CustomEvent(handled=False, args='args')",
                         self.buf.getvalue())

    def test_traces_hooks_created_before_own_registration(self):
        self.plugin_a.register()
        self.plugin_b.register()
        self.print_hooks_plugin.register()

        event = CustomEvent('args')
        result = self.session.hooks.pluginHookA(event)
        self.assertEqual(result, "TestPluginB.pluginHookA")
        self.assertEqual("\n"
                         "pluginHookA: "
                         "CustomEvent(handled=False, args='args')",
                         self.buf.getvalue())

    def test_traces_hooks_that_nobody_implements(self):
        self.plugin_a.register()
        self.print_hooks_plugin.register()

        event = CustomEvent('args')
        result = self.session.hooks.pluginHookA(event)
        self.assertEqual(result, None)
        self.assertEqual("\n"
                         "pluginHookA: "
                         "CustomEvent(handled=False, args='args')",
                         self.buf.getvalue())

    def test_indents_nested_hooks_in_trace(self):
        self.plugin_c = TestPluginC(session=self.session)

        self.plugin_a.register_with_nested_hook()
        self.plugin_c.register()
        self.print_hooks_plugin.register()

        event = CustomEvent('level_one_args')
        result = self.session.hooks.pluginHookB(event)
        self.assertEqual(result, "TestPluginC.pluginHookB")
        self.assertEqual("\n"
                         "pluginHookB: "
                         "CustomEvent(handled=False, args='level_one_args')"
                         "\n  "
                         "pluginHookB1: "
                         "CustomEvent(handled=False, args='level_two_args')",
                         self.buf.getvalue())

    def test_hook_implementors_can_modify_trace_indent(self):
        self.indent_aware_plugin = IndentAwarePlugin(session=self.session)

        self.plugin_a.register()
        self.indent_aware_plugin.register()
        self.print_hooks_plugin.register()

        event = CustomEvent('args')
        result = self.session.hooks.pluginHookA(event)
        self.assertEqual(result, "  ")



"""
This plugin implements :func:`startTestRun`, which excludes all test objects
that define a ``__test__`` attribute that evaluates to ``False``.
"""
from unittest import TestSuite
from nose2 import events

__unittest = True


class DunderTestFilter(events.Plugin):
    """
    Exclude all tests defining a ``__test__`` attribute that evaluates to ``False``.
    """
    alwaysOn = True

    def startTestRun(self, event):
        """
        Recurse :attr:`event.suite` and remove all test suites and test cases
        that define a ``__test__`` attribute that evaluates to ``False``.
        """
        self.removeNonTests(event.suite)

    def removeNonTests(self, suite):
        for test in list(suite):
            if not getattr(test, '__test__', True):
                suite._tests.remove(test)
            elif isinstance(test, TestSuite):
                self.removeNonTests(test)



"""
Unit tests for nose2 decorators.
"""

from nose2.tests._common import TestCase
from nose2.tools.decorators import with_setup, with_teardown


class WithSetupDecoratorTests(TestCase):
    def fake_setup(self):
        pass

    class fake_test(object):
        setup = None

    def test_setup_injection(self):
        sut = self.fake_test()
        expected = with_setup(self.fake_setup)(sut).setup

        self.assertEquals(expected, self.fake_setup)
        
        
class WithTeardownDecoratorTests(TestCase):
    def fake_teardown(self):
        pass

    class fake_test(object):
        teardown = None

    def test_teardown_injection(self):
        sut = self.fake_test()
        expected = with_teardown(self.fake_teardown)(sut).tearDownFunc

        self.assertEquals(expected, self.fake_teardown)



import time
import unittest

import logging

log = logging.getLogger(__name__)


class TestSlow(unittest.TestCase):

    def test_ok(self):
        print("hide this")
        time.sleep(2)

    def test_fail(self):
        print("show this")
        log.debug("hola")
        time.sleep(2)
        self.assertEqual(1, 2)

    def test_err(self):
        print("show this too")
        log.debug("ciao")
        time.sleep(2)
        {}['x']



"""Test testid plugin."""
import os.path
import pickle

from six import StringIO

from nose2 import session
from nose2.events import ReportTestEvent
from nose2.plugins import testid
from nose2.tests._common import (FakeStartTestEvent, FakeLoadFromNameEvent,
                                 FakeLoadFromNamesEvent, TestCase)


class UnitTestTestId(TestCase):

    """Test class TestId.

    Tests are carried out in a temporary directory, since TestId stores state
    to file. The temporary directory is removed after testing.
    """
    tags = ['unit']
    _RUN_IN_TEMP = True

    def setUp(self):
        super(UnitTestTestId, self).setUp()
        self.stream = StringIO()
        self.session = session.Session()
        self.plugin = testid.TestId(session=self.session)

    def test___init__(self):
        """Test the __init__ method."""
        plug = self.plugin
        # Test attributes
        for name, exp_val in [(
            'configSection', 'testid'), ('commandLineSwitch',
                                         ('I', 'with-id', 'Add test ids to output')), ('idfile',
                                                                                       os.path.abspath(
                                                                                           '.noseids')), ('ids', {}), ('tests', {}),
                ('id', 0)]:
            try:
                val = getattr(plug, name)
            except AttributeError:
                self.fail(
                    'TestId instance doesn\'t have attribute %s' % (name,))
            self.assertEqual(val, exp_val, 'Attribute %s should have value '
                             '\'%s\', but has value %s' % (name, exp_val, val))

    def test_start_test(self):
        """Test reportStartTest method."""
        self.session.verbosity = 2
        event = ReportTestEvent(FakeStartTestEvent(self), self.stream)
        plug = self.plugin
        plug.reportStartTest(event)

        self.assertEqual(plug.id, 1)
        test_id = self.id()
        self.assertEqual(plug.ids, {1: test_id})
        self.assertEqual(plug.tests, {test_id: 1})
        self.assertEqual(self.stream.getvalue(), '#1 ')

    def test_start_test_twice(self):
        """Test calling reportStartTest twice."""
        self.session.verbosity = 2
        event = ReportTestEvent(FakeStartTestEvent(self), self.stream)
        plug = self.plugin
        plug.reportStartTest(event)
        plug.reportStartTest(event)

        self.assertEqual(plug.id, 1)
        test_id = self.id()
        self.assertEqual(plug.ids, {1: test_id})
        self.assertEqual(plug.tests, {test_id: 1})
        self.assertEqual(self.stream.getvalue(), '#1 #1 ')

    def test_stop_test_run(self):
        """Test stopTestRun method."""
        plug = self.plugin
        plug.reportStartTest(
            ReportTestEvent(FakeStartTestEvent(self), self.stream))
        plug.stopTestRun(None)

        fh = open(plug.idfile, 'rb')
        try:
            data = pickle.load(fh)
        finally:
            fh.close()
        self.assertEqual(data, {'ids': plug.ids, 'tests': plug.tests})

    def test_load_tests_from_name(self):
        """Test loadTestsFromName method."""
        plug = self.plugin
        # By first starting/stopping a test, an ID is assigned by the plugin
        plug.reportStartTest(
            ReportTestEvent(FakeStartTestEvent(self), self.stream))
        plug.stopTestRun(None)
        event = FakeLoadFromNameEvent('1')
        plug.loadTestsFromName(event)

        # The numeric ID should be translated to this test's ID
        self.assertEqual(event.name, self.id())

    def test_load_tests_from_name_no_ids(self):
        """Test calling loadTestsFromName when no IDs have been saved."""
        plug = self.plugin
        event = FakeLoadFromNameEvent('1')
        plug.loadTestsFromName(event)

        # The event's name should be unchanged, since no IDs should be mapped
        self.assertEqual(event.name, '1')

    def test_load_tests_from_names(self):
        """Test loadTestsFromNames method."""
        plug = self.plugin
        # By first starting/stopping a test, an ID is assigned by the plugin
        plug.reportStartTest(
            ReportTestEvent(FakeStartTestEvent(self), self.stream))
        plug.stopTestRun(None)
        event = FakeLoadFromNamesEvent(['1', '2'])
        plug.loadTestsFromNames(event)

        name1, name2 = event.names
        # The first numeric ID should be translated to this test's ID
        self.assertEqual(name1, self.id())
        # The second one should not have a match
        self.assertEqual(name2, '2')



from nose2.compat import unittest

class TestWithExpectedFailures(unittest.TestCase):

    @unittest.expectedFailure
    def test_should_fail(self):
        assert False
    
    @unittest.expectedFailure
    def test_should_pass(self):
        assert True
    
    def test_whatever(self):
        assert True
    
    def test_fails(self):
        assert False



from nose2.tools import such
from nose2.tools.params import params

with such.A('foo') as it:
    @it.should('do bar')
    @params(1,2,3)
    def test(case, bar):
        case.assert_(isinstance(bar, int))

    @it.should('do bar and extra')
    @params((1, 2), (3, 4) ,(5, 6))
    def testExtraArg(case, bar, foo):
        case.assert_(isinstance(bar, int))
        case.assert_(isinstance(foo, int))

it.createTests(globals())



"""
This module contains some code copied from :mod:`unittest2` and other
code developed in reference to :mod:`unittest2`.

unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
Rights Reserved. See: http://docs.python.org/license.html

"""
import itertools

__unittest = True


def cartesian_params(*paramList):
    """Make a test function or method parameterized by cartesian product
    of parameters

    .. code-block :: python

      import unittest

      from nose2.tools import cartesian_params


      @cartesian_params((1, 2, 3), ('a', 'b'))
      def test_nums(num, char):
          assert num < ord(char)


      class Test(unittest.TestCase):

          @cartesian_params((1, 2, 3), ('a', 'b'))
          def test_less_than(self, num, char):
              self.assertLess(num, ord(char))

    Parameters in the list must be defined as iterable objects (such as
    ``tuple`` or ``list``).

    """
    def decorator(func):
        func.paramList = itertools.product(*paramList)
        return func
    return decorator


def params(*paramList):
    """Make a test function or method parameterized by parameters.

    .. code-block :: python

      import unittest

      from nose2.tools import params


      @params(1, 2, 3)
      def test_nums(num):
          assert num < 4


      class Test(unittest.TestCase):

          @params((1, 2), (2, 3), (4, 5))
          def test_less_than(self, a, b):
              assert a < b

    Parameters in the list may be defined as simple values, or as
    tuples. To pass a tuple as a simple value, wrap it in another tuple.

    """
    def decorator(func):
        func.paramList = paramList
        return func
    return decorator



from nose2.tests._common import FunctionalTestCase, support_file


class TestDoctestsPlugin(FunctionalTestCase):

    def test_simple(self):
        proc = self.runIn(
            'scenario/doctests',
            '-v',
            '--plugin=nose2.plugins.doctests',
            '--with-doctest',
        )
        self.assertTestRunOutputMatches(proc, stderr='Ran 6 tests')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs.rst ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs.txt ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: doctests_pkg1.docs1 ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs1.rst ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs1.txt ... ok')
        self.assertEqual(proc.poll(), 0)

    def test_start_directory_inside_package(self):
        proc = self.runIn(
            'scenario/doctests/doctests_pkg1',
            '-v',
            '--plugin=nose2.plugins.doctests',
            '--with-doctest',
            '-t',
            support_file('scenario/doctests')
        )
        self.assertTestRunOutputMatches(proc, stderr='Ran 3 tests')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: doctests_pkg1.docs1 ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs1.rst ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs1.txt ... ok')
        self.assertEqual(proc.poll(), 0)

    def test_project_directory_inside_package(self):
        proc = self.runIn(
            'scenario/doctests/doctests_pkg1',
            '-v',
            '--plugin=nose2.plugins.doctests',
            '--with-doctest',
        )
        self.assertTestRunOutputMatches(proc, stderr='Ran 3 tests')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: doctests_pkg1.docs1 ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs1.rst ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Doctest: docs1.txt ... ok')
        self.assertEqual(proc.poll(), 0)



"""unittest/unittest2 compatibility wrapper.

Anything internal to nose2 *must* import unittest from here to be
certain that it is using unittest2 when run on older Python versions.

Yes::

  from nose2.compat import unittest

**NO**::

  import unittest

**NO**::

  import unittest2

"""
try:
    import unittest2 as unittest
except ImportError:
    import unittest


try:
    unittest.installHandler
except AttributeError:
    raise ImportError(
        "Built-in unittest version too old; unittest2 is required")

__unittest = True


try:
    from collections import OrderedDict
except ImportError:
    from .backports.ordereddict import OrderedDict



import os.path
import platform
from nose2.compat import unittest

from nose2.tests._common import FunctionalTestCase


class TestCoverage(FunctionalTestCase):
    @unittest.skipIf(
        platform.python_version_tuple()[:2] == ('3', '2'),
        'coverage package does not support python 3.2')
    def test_run(self):
        proc = self.runIn(
            'scenario/test_with_module',
            '-v',
            '--with-coverage',
            '--coverage=lib/'
        )
        STATS = '\s+8\s+5\s+38%'
        expected = os.path.join('lib', 'mod1(.py)?')
        expected = expected.replace('\\', r'\\')
        expected = expected + STATS
        stdout, stderr = proc.communicate()
        self.assertTestRunOutputMatches(
            proc,
            stderr=expected)
        self.assertTestRunOutputMatches(
            proc,
            stderr='TOTAL\s+' + STATS)



import logging
log = logging.getLogger(__name__)

from nose2.tools import such

with such.A('foo') as it:
    it.upper_run = 0
    it.lower_run = 0
    
    @it.has_test_setup
    def upper_test_setup(case):
        log.error('foo::setUp')
        it.upper_run += 1
 
    @it.should("run upper setups")
    def test_run_upper_setups(case):
        case.assertEquals(it.upper_run, 1)
        case.assertEquals(it.lower_run, 0)
        
    with it.having('some bar'):
        @it.has_test_setup
        def lower_test_setup(case):
            log.error('foo some bar::setUp')
            it.lower_run += 1
 
        @it.should("run all setups")
        def test_run_all_setups(case):
            case.assertEquals(it.upper_run, 2)
            case.assertEquals(it.lower_run, 1)

        @it.should("run all setups again")
        def test_run_all_setups_again(case):
            case.assertEquals(it.upper_run, 3)
            case.assertEquals(it.lower_run, 2)

it.createTests(globals())



try:
    from unittest import mock
except ImportError:
    # Older python versions dont have mock by default
    import mock

from nose2.tests._common import TestCase, RedirectStdStreams
from nose2 import collector
from textwrap import dedent
import re


class TestCollector(TestCase):
    _RUN_IN_TEMP = True
    tags = ['unit']

    def test_collector_completes_with_no_tests(self):
        with open("unittest.cfg", "w") as ut_file:
            ut_file.write(dedent("""
                [unittest]
                quiet = true
            """))
        test = collector.collector()
        with RedirectStdStreams() as redir:
            self.assertRaises(SystemExit, test.run, None)
        self.assertEqual("", redir.stdout.getvalue())
        self.assertTrue(re.match(r'\n-+\nRan 0 tests in \d.\d\d\ds\n\nOK\n',
                                 redir.stderr.getvalue()))

    def test_collector_sets_testLoader_in_session(self):
        """
        session.testLoader needs to be set so that plugins that use this
        field (like Layers) dont break.
        """
        test = collector.collector()
        mock_session = mock.MagicMock()
        mock_loader = mock.MagicMock()
        mock_runner = mock.MagicMock()
        test._get_objects = mock.Mock(return_value=(mock_session,
                                                    mock_loader,
                                                    mock_runner))
        test._collector(None)
        self.assertTrue(mock_session.testLoader is mock_loader)



from contextlib import contextmanager
import inspect
import logging
import sys

import six

from nose2.compat import unittest
from nose2 import util
from nose2.main import PluggableTestProgram

log = logging.getLogger(__name__)

__unittest = True

LAYERS_PLUGIN_NOT_LOADED_MESSAGE = 'Warning: Such will not function properly if the "nose2.plugins.layers" plugin not loaded!\n'

@contextmanager
def A(description):
    """Test scenario context manager.

    Returns a :class:`nose2.tools.such.Scenario` instance,
    which by convention is bound to ``it``:

    .. code-block :: python

      with such.A('test scenario') as it:
          # tests and fixtures

    """
    yield Scenario(description)


class Helper(unittest.TestCase):

    def runTest(self):
        pass


helper = Helper()


class Scenario(object):

    """A test scenario.

    A test scenario defines a set of fixtures and tests
    that depend on those fixtures.
    """
    _helper = helper

    def __init__(self, description):
        self._group = Group('A %s' % description, 0)

    @contextmanager
    def having(self, description):
        """Define a new group under the current group.

        Fixtures and tests defined within the block will
        belong to the new group.

        .. code-block :: python

           with it.having('a description of this group'):
               # ...

        """
        last = self._group
        self._group = self._group.child(
            "having %s" % description)
        log.debug("starting new group from %s", description)
        yield self
        log.debug("leaving group %s", description)
        self._group = last

    def uses(self, layer):
        log.debug("Adding %s as mixin to %s", layer, self._group)
        self._group.mixins.append(layer)

    def has_setup(self, func):
        """Add a :func:`setup` method to this group.

        The :func:`setup` method will run once, before any of the
        tests in the containing group.

        A group may define any number of :func:`setup` functions. They
        will execute in the order in which they are defined.

        .. code-block :: python

           @it.has_setup
           def setup():
               # ...

        """
        self._group.addSetup(func)
        return func

    def has_teardown(self, func):
        """Add a :func:`teardown` method to this group.

        The :func:`teardown` method will run once, after all of the
        tests in the containing group.

        A group may define any number of :func:`teardown` functions. They
        will execute in the order in which they are defined.

        .. code-block :: python

           @it.has_teardown
           def teardown():
               # ...

        """
        self._group.addTeardown(func)
        return func

    def has_test_setup(self, func):
        """Add a test case :func:`setup` method to this group.

        The :func:`setup` method will run before each of the
        tests in the containing group.

        A group may define any number of test case :func:`setup`
        functions. They will execute in the order in which they are
        defined.

        Test :func:`setup` functions may optionally take one argument. If
        they do, they will be passed the :class:`unittest.TestCase`
        instance generated for the test.

        .. code-block :: python

           @it.has_test_setup
           def setup(case):
               # ...

        """
        self._group.addTestSetUp(func)

    def has_test_teardown(self, func):
        """Add a test case :func:`teardown` method to this group.

        The :func:`teardown` method will run before each of the
        tests in the containing group.

        A group may define any number of test case :func:`teardown`
        functions. They will execute in the order in which they are
        defined.

        Test :func:`teardown` functions may optionally take one argument. If
        they do, they will be passed the :class:`unittest.TestCase`
        instance generated for the test.

        .. code-block :: python

           @it.has_test_teardown
           def teardown(case):
               # ...

        """
        self._group.addTestTearDown(func)

    def should(self, desc):
        """Define a test case.

        Each function marked with this decorator becomes a test
        case in the current group.

        The decorator takes one optional argument, the description
        of the test case: what it **should** do. If this argument
        is not provided, the docstring of the decorated function
        will be used as the test case description.

        Test functions may optionally take one argument. If they do,
        they will be passed the :class:`unittest.TestCase` instance generated
        for the test. They can use this TestCase instance to execute assert
        methods, among other things.

        .. code-block :: python

           @it.should('do this')
           def dothis(case):
               # ....

           @it.should
           def dothat():
               "do that also"
               # ....

        """
        def decorator(f):
            _desc = desc if isinstance(desc, six.string_types) else f.__doc__
            case = Case(self._group, f, "should %s" % _desc)
            self._group.addCase(case)
            return case
        if isinstance(desc, type(decorator)):
            return decorator(desc)
        return decorator

    def __getattr__(self, attr):
        return getattr(self._helper, attr)

    def createTests(self, mod):
        """Generate test cases for this scenario.

        .. warning ::

           You must call this, passing in :func:`globals`, to
           generate tests from the scenario. If you don't,
           **no tests will be created**.

        .. code-block :: python

           it.createTests(globals())

        """
        self._checkForLayersPlugin()
        self._makeGroupTest(mod, self._group)

    def _checkForLayersPlugin(self):
        currentSession = PluggableTestProgram.getCurrentSession()
        if not currentSession:
            return
        if not currentSession.isPluginLoaded('nose2.plugins.layers'):
            sys.stderr.write(LAYERS_PLUGIN_NOT_LOADED_MESSAGE)

    def _makeGroupTest(self, mod, group, parent_layer=None, position=0):
        layer = self._makeLayer(group, parent_layer, position)
        case = self._makeTestCase(group, layer, parent_layer)
        log.debug(
            "Made test case %s with layer %s from %s", case, layer, group)
        mod[layer.__name__] = layer
        layer.__module__ = mod['__name__']
        name = case.__name__
        long_name = ' '.join(
            [n[0].description for n in util.ancestry(layer)] + [name])
        mod[long_name] = case
        if name not in mod:
            mod[name] = case
        case.__module__ = mod['__name__']
        for index, child in enumerate(group._children):
            self._makeGroupTest(mod, child, layer, index)

    def _makeTestCase(self, group, layer, parent_layer):
        attr = {
            'layer': layer,
            'group': group,
            'description': group.description,
        }

        def _make_test_func(case):
            '''
            Needs to be outside of the for-loop scope, so that ``case`` is properly registered as a closure.
            '''
            def _test(s, *args):
                case(s, *args)
            return _test
            
        for index, case in enumerate(group._cases):
            name = 'test %04d: %s' % (index, case.description)
            _test = _make_test_func(case)
            _test.__name__ = name
            _test.description = case.description
            _test.case = case
            _test.index = index
            if hasattr(case.func, 'paramList'):
                _test.paramList = case.func.paramList
            attr[name] = _test  # for collection and sorting
            attr[case.description] = _test  # for random access by name

        setups = getattr(parent_layer, 'testSetups', []) + group._test_setups
        if setups:
            def setUp(self):
                for func in setups:
                    args, _, _, _ = inspect.getargspec(func)
                    if args:
                        func(self)
                    else:
                        func()
            attr['setUp'] = setUp
        teardowns = getattr(parent_layer, 'testTeardowns', []) + group._test_teardowns[:]
        if teardowns:
            def tearDown(self):
                for func in teardowns:
                    args, _, _, _ = inspect.getargspec(func)
                    if args:
                        func(self)
                    else:
                        func()
            attr['tearDown'] = tearDown

        def methodDescription(self):
            return getattr(self, self._testMethodName).description
        attr['methodDescription'] = methodDescription

        return type(group.description, (unittest.TestCase,), attr)

    def _makeLayer(self, group, parent_layer=None, position=0):
        if parent_layer is None:
            parent_layer = object

        def setUp(cls):
            for func in cls.setups:
                args, _, _, _ = inspect.getargspec(func)
                if args:
                    func(self)
                else:
                    func()

        def tearDown(cls):
            for func in cls.teardowns:
                args, _, _, _ = inspect.getargspec(func)
                if args:
                    func(self)
                else:
                    func()

        attr = {
            'description': group.description,
            'setUp': classmethod(setUp),
            'tearDown': classmethod(tearDown),
            'setups': group._setups[:],
            'testSetups': getattr(parent_layer, 'testSetups', []) + group._test_setups,
            'teardowns': group._teardowns[:],
            'testTeardowns': getattr(parent_layer, 'testTeardowns', []) + group._test_teardowns[:],
            'position': position,
            'mixins': ()
        }

        if group.base_layer:
            # inject this layer into the group class list
            # by making it a subclass of parent_layer
            layer = group.base_layer
            if parent_layer not in layer.__bases__:
                layer.mixins = (parent_layer,)
        else:
            layer = type("%s:layer" % group.description, (parent_layer,), attr)
        if group.mixins:
            layer.mixins = getattr(layer, 'mixins', ()) + tuple(group.mixins)
        log.debug("made layer %s with bases %s and mixins %s",
                  layer, layer.__bases__, layer.mixins)
        return layer


class Group(object):

    """A group of tests, with common fixtures and description"""

    def __init__(self, description, indent=0, parent=None, base_layer=None):
        self.description = description
        self.indent = indent
        self.parent = parent
        self.base_layer = base_layer
        self.mixins = []
        self._cases = []
        self._setups = []
        self._teardowns = []
        self._test_setups = []
        self._test_teardowns = []
        self._children = []

    def addCase(self, case):
        if not self._cases:
            case.first = True
        case.indent = self.indent
        self._cases.append(case)

    def addSetup(self, func):
        self._setups.append(func)

    def addTeardown(self, func):
        self._teardowns.append(func)

    def addTestSetUp(self, func):
        self._test_setups.append(func)

    def addTestTearDown(self, func):
        self._test_teardowns.append(func)

    def fullDescription(self):
        d = []
        p = self.parent
        while p:
            d.insert(0, p.description)
            p = p.parent
        d.append(self.description)
        return ' '.join(d)

    def child(self, description, base_layer=None):
        child = Group(description, self.indent + 1, self, base_layer)
        self._children.append(child)
        return child


class Case(object):

    """Information about a test case"""
    _helper = helper

    def __init__(self, group, func, description):
        self.group = group
        self.func = func
        self.description = description
        self._setups = []
        self._teardowns = []
        self.first = False
        self.full = False

    def __call__(self, testcase, *args):
        # ... only if it takes an arg
        self._helper = testcase
        funcargs, _, _, _ = inspect.getargspec(self.func)
        if funcargs:
            self.func(testcase, *args)
        else:
            self.func()

    def __getattr__(self, attr):
        return getattr(self._helper, attr)



import unittest


def load_tests(loader, standard_tests, pattern):
    suite = loader.suiteClass()

    class Test(unittest.TestCase):

        def test(self):
            import ltpkg2
            assert ltpkg2.lt(1, 2)
    suite.addTest(Test('test'))
    return suite



from nose2.compat import unittest


class L1(object):

    @classmethod
    def setUp(cls):
        print('L1 setUp')

    @classmethod
    def testSetUp(cls):
        print('L1 testSetUp')

    @classmethod
    def tearDown(cls):
        print('L1 tearDown')

    @classmethod
    def testTearDown(cls):
        print('L1 testTearDown')


class L2(L1):

    @classmethod
    def setUp(cls):
        print('L2 setUp')

    @classmethod
    def testSetUp(cls):
        print('L2 testSetUp')

    @classmethod
    def testTearDown(cls):
        print('L2 testTearDown')


# L1 tearDown should only run once
class T1(unittest.TestCase):
    layer = L2

    def test1(self):
        print('Run test1')

    def test2(self):
        print('Run test2')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose2.tools import such

with such.A('system') as it:

    with it.having('something'):
        @it.should("do stuff")
        def test_should_do_stuff(case):
            pass

    it.createTests(globals())

with such.A('another system') as it:

    with it.having('something'):
        @it.should("do stuff")
        def test_should_do_stuff(case):
            pass

    it.createTests(globals())


from nose2 import events, loader, session, util
from nose2.plugins.loader import generators, testcases
from nose2.tests._common import TestCase


class TestGeneratorUnpack(TestCase):
    tags = ['unit']

    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(self.session)
        self.expect = [(0, ('call', (0, 1))),
                       (1, ('call', (1, 2))),
                       (2, ('call', (2, 3))), ]
        self.plugin = generators.Generators(session=self.session)
        # need testcase loader to make the initial response to load from module
        self.tcl = testcases.TestCaseLoader(session=self.session)

    def test_unpack_handles_nose_style_generators(self):
        def gen():
            for i in range(0, 3):
                yield 'call', i, i + 1
        out = list(self.plugin.unpack(gen()))
        self.assertEqual(out, self.expect)

    def test_unpack_handles_unittest2_style_generators(self):
        def gen():
            for i in range(0, 3):
                yield 'call', (i, i + 1)
        out = list(self.plugin.unpack(gen()))
        self.assertEqual(out, self.expect)

    def test_ignores_ordinary_functions(self):
        class Mod(object):
            pass

        def test():
            pass
        m = Mod()
        m.test = test
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(len(event.extraTests), 0)

    def test_can_load_tests_from_generator_functions(self):
        class Mod(object):
            __name__ = 'themod'

        def check(x):
            assert x == 1

        def test():
            yield check, 1
            yield check, 2
        m = Mod()
        m.test = test
        test.__module__ = m.__name__
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(len(event.extraTests), 2)
        # check that test names are sensible
        self.assertEqual(util.test_name(event.extraTests[0]),
                         'themod.test:1')
        self.assertEqual(util.test_name(event.extraTests[1]),
                         'themod.test:2')

    def test_can_load_tests_from_generator_methods(self):
        class Mod(object):
            pass

        def check(x):
            return x == 1

        class Test(TestCase):

            def test(self):
                yield check, 1
                yield check, 2
        m = Mod()
        m.Test = Test
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(len(event.extraTests), 1)
        self.assertEqual(len(event.extraTests[0]._tests), 2)



from nose2.tests._common import FunctionalTestCase
from nose2.tools import such

class TestSuchDSL(FunctionalTestCase):
    def test_runs_example(self):
        proc = self.runIn(
            'such',
            '-v',
            '--plugin=nose2.plugins.layers',
            'test_such')
        self.assertTestRunOutputMatches(proc, stderr='Ran 9 tests')
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())

    def test_load_top_level_by_name(self):
        proc = self.runIn(
            'such',
            '-v',
            '--plugin=nose2.plugins.layers',
            'test_such.A system with complex setup.should do something')
        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test')
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())

    def test_load_sublayer_test_by_name(self):
        proc = self.runIn(
            'such',
            '-v',
            '--plugin=nose2.plugins.layers',
            'test_such.having an expensive fixture.'
            'should do more things')
        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test')
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())

    def test_regression_tests_with_the_same_having_description_under_different_fixtures_in_the_same_module_should_be_run(
            self):
        proc = self.runIn(
            'such',
            '-v',
            '--plugin=nose2.plugins.layers',
            'test_regression_same_havings')
        self.assertTestRunOutputMatches(proc, stderr='Ran 2 test')
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())

    def test_teardown_fail(self):
        proc = self.runIn('scenario/layers_with_errors',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'test_such_teardown_fail')
        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test in')
        self.assertTestRunOutputMatches(proc, stderr='ERROR: LayerSuite')
        self.assertTestRunOutputMatches(proc, stderr=r'FAILED \(errors=1\)')
        self.assertTestRunOutputMatches(proc, stderr='Bad Error in such tearDown')

    def test_setup_fail(self):
        proc = self.runIn('scenario/layers_with_errors',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'test_such_setup_fail')
        self.assertTestRunOutputMatches(proc, stderr='Ran 0 tests in')
        self.assertTestRunOutputMatches(proc, stderr='ERROR: LayerSuite')
        self.assertTestRunOutputMatches(proc, stderr=r'FAILED \(errors=1\)')
        self.assertTestRunOutputMatches(proc, stderr='Bad Error in such setUp!')

    def test_param_plugin_with_such(self):
        proc = self.runIn('scenario/such_with_params',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'such_with_params')
        self.assertTestRunOutputMatches(proc, stderr='Ran 6 tests in')
        self.assertTestRunOutputMatches(proc, stderr='OK')
        self.assertTestRunOutputMatches(proc, stderr='test 0000: should do bar:3')
        self.assertTestRunOutputMatches(proc, stderr='test 0001: should do bar and extra:3')

    def test_such_without_layers_plugin(self):
        proc = self.runIn('such',
                          '-v',
                          'test_such_without_layers')
        self.assertTestRunOutputMatches(proc, stderr=r'FAILED')
        self.assertTestRunOutputMatches(proc, stderr=such.LAYERS_PLUGIN_NOT_LOADED_MESSAGE)

    def test_testsetup_on_higher_layer_with_test(self):
        proc = self.runIn('scenario/layers_setups',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'higher_layer_testsetup_with_test')
        self.assertTestRunOutputMatches(proc, stderr='Ran 3 tests in')
        self.assertTestRunOutputMatches(proc, stderr='OK')

    def test_testsetup_on_higher_layer(self):
        proc = self.runIn('scenario/layers_setups',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'higher_layer_testsetup_no_test')
        self.assertTestRunOutputMatches(proc, stderr='Ran 2 tests in')
        self.assertTestRunOutputMatches(proc, stderr='OK')

    def test_testsetup_on_higher_layer_3layers(self):
        proc = self.runIn('scenario/layers_setups',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'higher_layer_testsetup_3layers')
        self.assertTestRunOutputMatches(proc, stderr='Ran 2 tests in')
        self.assertTestRunOutputMatches(proc, stderr='OK')

    def test_setup_on_higher_layer(self):
        proc = self.runIn('scenario/layers_setups',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'higher_layer_setup')
        self.assertTestRunOutputMatches(proc, stderr='Ran 2 tests in')
        self.assertTestRunOutputMatches(proc, stderr='OK')




from nose2 import session
from nose2.plugins import prof
from nose2.events import StartTestRunEvent
from nose2.tests._common import Stub, TestCase


class TestProfPlugin(TestCase):
    tags = ['unit']

    def setUp(self):
        self.plugin = prof.Profiler(session=session.Session())
        self.hotshot = prof.hotshot
        self.stats = prof.stats
        prof.hotshot = Stub()
        prof.stats = Stub()

    def tearDown(self):
        prof.hotshot = self.hotshot
        prof.stats = self.stats

    def test_startTestRun_sets_executeTests(self):
        _prof = Stub()
        _prof.runcall = object()
        prof.hotshot.Profile = lambda filename: _prof
        event = StartTestRunEvent(runner=None, suite=None, result=None,
                                  startTime=None, executeTests=None)
        self.plugin.startTestRun(event)
        assert event.executeTests is _prof.runcall, \
            "executeTests was not replaced"



"""
Profile test execution using hotshot.

This plugin implements :func:`startTestRun` and replaces
``event.executeTests`` with :meth:`hotshot.Profile.runcall`. It
implements :func:`beforeSummaryReport` to output profiling information
before the final test summary time. Config file options ``filename``,
``sort`` and ``restrict`` can be used to change where profiling
information is saved and how it is presented.

"""
try:
    import hotshot
    from hotshot import stats
except ImportError:
    hotshot, stats = None, None
import logging
import os
import tempfile

from nose2 import events, util

log = logging.getLogger(__name__)
__unittest = True


class Profiler(events.Plugin):

    """Profile the test run"""

    configSection = 'profiler'
    commandLineSwitch = ('P', 'profile', 'Run tests under profiler')

    def __init__(self):
        self.pfile = self.config.as_str('filename', '')
        self.sort = self.config.as_str('sort', 'cumulative')
        self.restrict = self.config.as_list('restrict', [])
        self.clean = False
        self.fileno = None

    def register(self):
        """Don't register if hotshot is not found"""
        if hotshot is None:
            log.error("Unable to profile: hotshot module not available")
            return
        super(Profiler, self).register()

    def startTestRun(self, event):
        """Set up the profiler"""
        self.createPfile()
        self.prof = hotshot.Profile(self.pfile)
        event.executeTests = self.prof.runcall

    def beforeSummaryReport(self, event):
        """Output profiling results"""
        # write prof output to stream
        class Stream:

            def write(self, *msg):
                for m in msg:
                    event.stream.write(m)
                    event.stream.write(' ')
                event.stream.flush()
        stream = Stream()
        self.prof.close()
        prof_stats = stats.load(self.pfile)
        prof_stats.sort_stats(self.sort)
        event.stream.writeln(util.ln("Profiling results"))
        tmp = prof_stats.stream
        prof_stats.stream = stream
        try:
            if self.restrict:
                prof_stats.print_stats(*self.restrict)
            else:
                prof_stats.print_stats()
        finally:
            prof_stats.stream = tmp
        self.prof.close()
        event.stream.writeln('')

        if self.clean:
            if self.fileno:
                try:
                    os.close(self.fileno)
                except OSError:
                    pass
            try:
                os.unlink(self.pfile)
            except OSError:
                pass

    def createPfile(self):
        if not self.pfile:
            self.fileno, self.pfile = tempfile.mkstemp()
            self.clean = True



from nose2 import session
from nose2.tests._common import TestCase, Conn
from nose2.plugins import mp
import sys


class TestMPPlugin(TestCase):

    def setUp(self):
        self.session = session.Session()
        self.plugin = mp.MultiProcess(session=self.session)

    def test_gentests(self):
        conn = Conn([1, 2, 3])
        res = []
        for x in mp.gentests(conn):
            res.append(x)
        self.assertEqual(res, [1, 2, 3])

    def test_recording_plugin_interface(self):
        rpi = mp.RecordingPluginInterface()
        # this one should record
        rpi.setTestOutcome(None)
        # none of these should record
        rpi.getTestCaseNames(None)
        rpi.startSubprocess(None)
        rpi.stopSubprocess(None)
        rpi.registerInSubprocess(None)
        rpi.loadTestsFromModule(None)
        rpi.loadTestsFromTestCase(None)
        self.assertEqual(rpi.flush(), [('setTestOutcome', None)])

    def test_address(self):
        platform = sys.platform
        try:
            sys.platform = "linux"
            host = "1.2.3.4"
            port = 245
            self.plugin.setAddress(host)
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port),
                             (host, 0))
            self.plugin.setAddress("%s:%i" % (host, port))
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port),
                             (host, port))
            self.plugin.setAddress(None)
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port),
                             (None, 0))
            sys.platform = "win32"
            self.plugin.setAddress(host)
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port),
                             (host, 0))
            self.plugin.setAddress("%s:%i" % (host, port))
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port),
                             (host, port))
            self.plugin.setAddress(None)
            self.assertEqual((self.plugin.bind_host, self.plugin.bind_port),
                             ("127.116.157.163", 0))
        finally:
            sys.platform = platform




from nose2.tests._common import FunctionalTestCase


class DecoratorsTests(FunctionalTestCase):
    def test_with_setup(self):
        process = self.runIn(
            'scenario/decorators', 'test_decorators.test_with_setup')

        self.assertTestRunOutputMatches(process, stderr="Ran 1 test")
        self.assertEqual(process.poll(), 0, process.stderr.getvalue())
        
    def test_with_teardown(self):
        process = self.runIn(
            'scenario/decorators', 'test_decorators.test_with_teardown', 'test_decorators.test_teardown_ran')

        self.assertTestRunOutputMatches(process, stderr="Ran 2 test")
        self.assertEqual(process.poll(), 0, process.stderr.getvalue())



from nose2.tests._common import TestCase
from nose2.plugins.loader.testcases import TestCaseLoader
from nose2 import events, loader, session


class TestTestCaseLoader(TestCase):

    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(session=self.session)
        self.plugin = TestCaseLoader(session=self.session)

        class Mod(object):
            pass
        self.module = Mod()

        class A(TestCase):

            def test(self):
                pass

        class B(TestCase):

            def runTest(self):
                pass

        class C(TestCase):

            def foo(self):
                pass

        class Test(object):

            def test(self):
                pass

        self.module.A = A
        self.module.B = B
        self.module.C = C
        self.module.Test = Test

    def test_can_find_testcases_in_module(self):
        event = events.LoadFromModuleEvent(self.loader, self.module)
        result = self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(result, None)
        self.assertEqual(len(event.extraTests), 3)
        self.assertEqual(len(event.extraTests[0]._tests), 1)  # A
        self.assertEqual(len(event.extraTests[1]._tests), 1)  # B
        self.assertEqual(len(event.extraTests[2]._tests), 0)  # C

    def test_get_testcase_names_can_override_name_selection(self):
        class FooIsOnlyTest(events.Plugin):

            def getTestCaseNames(self, event):
                event.handled = True
                return ['foo'] if 'foo' in dir(event.testCase) else []
        foo = FooIsOnlyTest(session=self.session)
        foo.register()
        event = events.LoadFromModuleEvent(self.loader, self.module)
        result = self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(result, None)
        self.assertEqual(len(event.extraTests), 3)
        self.assertEqual(len(event.extraTests[0]._tests), 0)  # A
        self.assertEqual(len(event.extraTests[1]._tests), 1)  # B (runTest)
        self.assertEqual(len(event.extraTests[2]._tests), 1)  # C

    def test_plugins_can_exclude_test_names(self):
        class Excluder(events.Plugin):

            def getTestCaseNames(self, event):
                event.excludedNames.append('test')
        excl = Excluder(session=self.session)
        excl.register()
        event = events.LoadFromModuleEvent(self.loader, self.module)
        result = self.session.hooks.loadTestsFromModule(event)
        self.assertEqual(result, None)
        self.assertEqual(len(event.extraTests), 3)
        self.assertEqual(len(event.extraTests[0]._tests), 0)  # A
        self.assertEqual(len(event.extraTests[1]._tests), 1)  # B (runTest)
        self.assertEqual(len(event.extraTests[2]._tests), 0)  # C



from nose2.tests._common import FunctionalTestCase, TestCase, support_file
from nose2 import events, loader, session
from nose2.plugins.loader.discovery import DiscoveryLoader


class Watcher(events.Plugin):

    def __init__(self):
        self.called = []

    def loadTestsFromModule(self, event):
        self.called.append(event)


class DiscoveryFunctionalTest(FunctionalTestCase):

    def setUp(self):
        self.session = session.Session()
        self.plug = DiscoveryLoader(session=self.session)
        self.loader = loader.PluggableTestLoader(self.session)
        self.watcher = Watcher(session=self.session)
        self.watcher.register()

    def test_can_discover_test_modules_in_packages(self):
        self.session.startDir = support_file('scenario/tests_in_package')
        event = events.LoadFromNamesEvent(self.loader, [], None)
        result = self.session.hooks.loadTestsFromNames(event)
        assert isinstance(result, self.loader.suiteClass)
        self.assertEqual(len(result._tests), 1)
        self.assertEqual(len(self.watcher.called), 1)
        self.assertEqual(self.watcher.called[0].module.__name__,
                         'pkg1.test.test_things')

    def test_discovery_supports_code_in_lib_dir(self):
        self.session.startDir = support_file('scenario/package_in_lib')
        event = events.LoadFromNamesEvent(self.loader, [], None)
        result = self.session.hooks.loadTestsFromNames(event)
        assert isinstance(result, self.loader.suiteClass)
        self.assertEqual(len(result._tests), 1)
        self.assertEqual(len(self.watcher.called), 1)
        self.assertEqual(self.watcher.called[0].module.__name__, 'tests')

    def test_match_path_event_can_prevent_discovery(self):
        class NoTestsForYou(events.Plugin):

            def matchPath(self, event):
                event.handled = True
                return False
        mp = NoTestsForYou(session=self.session)
        mp.register()
        self.session.startDir = support_file('scenario/tests_in_package')
        event = events.LoadFromNamesEvent(self.loader, [], None)
        result = self.session.hooks.loadTestsFromNames(event)
        assert isinstance(result, self.loader.suiteClass)
        self.assertEqual(len(result._tests), 0)
        self.assertEqual(len(self.watcher.called), 0)

    def test_handle_file_event_can_add_tests(self):
        class TextTest(TestCase):

            def test(self):
                pass

        class TestsInText(events.Plugin):

            def handleFile(self, event):
                if event.path.endswith('.txt'):
                    event.extraTests.append(TextTest('test'))
        mp = TestsInText(session=self.session)
        mp.register()
        self.session.startDir = support_file('scenario/tests_in_package')
        event = events.LoadFromNamesEvent(self.loader, [], None)
        result = self.session.hooks.loadTestsFromNames(event)
        assert isinstance(result, self.loader.suiteClass)
        self.assertEqual(len(result._tests), 2)
        self.assertEqual(len(self.watcher.called), 1)



import unittest


class Test(unittest.TestCase):

    def test_fast(self):
        pass
    test_fast.fast = 1
    test_fast.layer = 2
    test_fast.flags = ['blue', 'green']

    def test_faster(self):
        pass
    test_faster.fast = 1
    test_faster.layer = 1
    test_faster.flags = ['red', 'green']

    def test_slow(self):
        pass
    test_slow.fast = 0
    test_slow.slow = 1
    test_slow.layer = 2

    def test_slower(self):
        pass
    test_slower.slow = 1
    test_slower.layer = 3
    test_slower.flags = ['blue', 'red']



from nose2.tests._common import FunctionalTestCase


class TestAttribPlugin(FunctionalTestCase):

    def test_simple_true(self):
        proc = self.runIn(
            'scenario/tests_in_package',
            '-v',
            '--plugin=nose2.plugins.attrib',
            '-A',
            'a')
        self.assertTestRunOutputMatches(proc, stderr='Ran 4 tests')
        self.assertTestRunOutputMatches(proc, stderr='test_params_method')
        self.assertTestRunOutputMatches(proc, stderr='test_func')

    def test_simple_false(self):
        proc = self.runIn(
            'scenario/tests_in_package',
            '-v',
            '--plugin=nose2.plugins.attrib',
            '-A',
            '!a')
        self.assertTestRunOutputMatches(proc, stderr='Ran 21 tests')

    def test_simple_value(self):
        proc = self.runIn(
            'scenario/tests_in_package',
            '-v',
            '--plugin=nose2.plugins.attrib',
            '-A',
            'b=2')
        self.assertTestRunOutputMatches(proc, stderr='Ran 2 tests')

    def test_list_value(self):
        proc = self.runIn(
            'scenario/tests_in_package',
            '-v',
            '--plugin=nose2.plugins.attrib',
            '-A',
            'tags=func')
        self.assertTestRunOutputMatches(proc, stderr='Ran 8 tests')
        self.assertTestRunOutputMatches(proc, stderr='test_params_func')
        self.assertTestRunOutputMatches(proc, stderr='test_func')
        self.assertTestRunOutputMatches(proc, stderr='test_gen')

    def test_list_value_negation(self):
        proc = self.runIn(
            'scenario/tests_in_package',
            '-v',
            '--plugin=nose2.plugins.attrib',
            '-A',
            '!tags=func')
        self.assertTestRunOutputMatches(proc, stderr='Ran 8 tests')
        self.assertTestRunOutputMatches(proc, stderr='test_gen_method')
        self.assertTestRunOutputMatches(proc, stderr='test_params_method')
        self.assertTestRunOutputMatches(proc, stderr='test_ok')
        self.assertTestRunOutputMatches(proc, stderr='test_failed')
        self.assertTestRunOutputMatches(proc, stderr='test_skippy')
        self.assertTestRunOutputMatches(proc, stderr='test_typeerr')

    def test_eval_expr(self):
        proc = self.runIn(
            'scenario/tests_in_package',
            '-v',
            '--plugin=nose2.plugins.attrib',
            '-E',
            'a == b and a')
        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test')
        self.assertTestRunOutputMatches(proc, stderr='skippy')



import sys
from nose2.tests._common import FunctionalTestCase, support_file
import nose2.plugins.loader.eggdiscovery #@UnusedImport This is not very elegant, but it allows eggdiscovery to be found in Travis (or when run with PYTHONPATH=.)

try:
    import pkg_resources

except ImportError:
    pkg_resources = None

else:
    class EggDiscoveryFunctionalTest(FunctionalTestCase):
        def setUp(self):
            for m in [m for m in sys.modules if m.startswith('pkgegg')]:
                del sys.modules[m]
            self.egg_path = support_file('scenario/tests_in_zipped_eggs/pkgegg-0.0.0-py2.7.egg')
            sys.path.append(self.egg_path)
    
        def tearDown(self):
            if self.egg_path in sys.path:
                sys.path.remove(self.egg_path)
            for m in [m for m in sys.modules if m.startswith('pkgegg')]:
                del sys.modules[m]
        
        def test_non_egg_discoverer_does_not_fail_when_looking_in_egg(self):
            proc = self.runIn(
                'scenario/tests_in_zipped_eggs',
                '-v',
                'pkgegg')
            self.assertTestRunOutputMatches(proc, stderr='Ran 0 tests in')
        
        def test_can_discover_test_modules_in_zipped_eggs(self):
            proc = self.runIn(
                'scenario/tests_in_zipped_eggs', 
                '-v',
                '--plugin=nose2.plugins.loader.eggdiscovery',
                'pkgegg')
            self.assertTestRunOutputMatches(proc, stderr='FAILED \(failures=5, errors=1, skipped=1\)')
    
        def test_eggdiscovery_failure_does_not_exist(self):
            proc = self.runIn(
                'scenario', 
                '-v',
                '--plugin=nose2.plugins.loader.eggdiscovery',
                '--exclude-plugin=nose2.plugins.loader.discovery',
                '-s',
                'tests_in_zipped_eggs_BAD')
            self.assertTestRunOutputMatches(proc, stderr='tests_in_zipped_eggs_BAD does not exist')
    
    class UnzippedEggDiscoveryFunctionalTest(FunctionalTestCase):
        def setUp(self):
            for m in [m for m in sys.modules if m.startswith('pkgegg')]:
                del sys.modules[m]
            self.egg_path = support_file('scenario/tests_in_unzipped_eggs/pkgunegg-0.0.0-py2.7.egg')
            sys.path.append(self.egg_path)
    
        def tearDown(self):
            if self.egg_path in sys.path:
                sys.path.remove(self.egg_path)
            for m in [m for m in sys.modules if m.startswith('pkgunegg')]:
                del sys.modules[m]
        
        def test_eggdiscovery_ignores_unzipped_eggs(self):
            proc = self.runIn(
                'scenario/tests_in_unzipped_eggs', 
                '-v',
                '--plugin=nose2.plugins.loader.eggdiscovery',
                'pkgunegg')
            self.assertTestRunOutputMatches(proc, stderr='FAILED \(failures=5, errors=1, skipped=1\)')
    


from nose2.tools import such

with such.A('test scenario with errors') as it:

    @it.has_teardown
    def teardown_fail():
        raise RuntimeError('Bad Error in such tearDown!')

    @it.should('check that value == 1')
    def test_passes(case):
        pass

it.createTests(globals())



import unittest


class TestCase(unittest.TestCase):

    def test_a(self):
        pass

    def test_b(self):
        pass

    def test_c(self):
        pass


def load_tests(loader, tests, pattern):
    del tests._tests[0]._tests[1]
    return tests



from nose2.tests._common import TestCase, support_file
from nose2 import util


class UtilTests(TestCase):

    def test_name_from_path(self):
        test_module = support_file('scenario/tests_in_package/pkg1/test/test_things.py')
        test_package_path = support_file('scenario/tests_in_package')
        self.assertEqual(
            util.name_from_path(test_module),
            ('pkg1.test.test_things', test_package_path)
        )



from nose2.tools import such

with such.A('system with complex setup') as it:

    @it.has_setup
    def setup():
        it.things = [1]

    @it.has_teardown
    def teardown():
        it.things = []

    @it.should('do something')
    def test():
        assert it.things
        it.assertEqual(len(it.things), 1)

    with it.having('an expensive fixture'):
        @it.has_setup
        def setup():
            it.things.append(2)

        @it.should('do more things')
        def test(case):
            case.assertEqual(it.things[-1], 2)

it.createTests(globals())



"""Common functionality."""
import os.path
import tempfile
import shutil
import subprocess
import sys

import six

from nose2.compat import unittest
from nose2 import discover, util


HERE = os.path.abspath(os.path.dirname(__file__))
SUPPORT = os.path.join(HERE, 'functional', 'support')


class TestCase(unittest.TestCase):

    """TestCase extension.

    If the class variable ``_RUN_IN_TEMP`` is ``True`` (default: ``False``), tests will be
    performed in a temporary directory, which is deleted afterwards.
    """
    _RUN_IN_TEMP = False

    def setUp(self):
        super(TestCase, self).setUp()

        if self._RUN_IN_TEMP:
            self._orig_dir = os.getcwd()
            work_dir = self._work_dir = tempfile.mkdtemp()
            os.chdir(self._work_dir)
            # Make sure it's possible to import modules from current directory
            sys.path.insert(0, work_dir)

    def tearDown(self):
        super(TestCase, self).tearDown()

        if self._RUN_IN_TEMP:
            os.chdir(self._orig_dir)
            shutil.rmtree(self._work_dir, ignore_errors=True)

    def __str__(self):
        """
        In python 3.5, the unittest.TestCase.__str__() output changed.
        This makes it conform to previous version.
        """
        if sys.version_info >= (3, 5):
            test_module = self.__class__.__module__
            test_class = self.__class__.__name__
            test_method = self._testMethodName
            return "%s (%s.%s)" % (test_method, test_module, test_class)
        else:
            return super(TestCase, self).__str__()

    def id(self):
        """
        In python 3.5, the unittest.TestCase.__id__() output changed.
        This makes it conform to previous version.
        """
        if sys.version_info >= (3, 5):
            test_module = self.__class__.__module__
            test_class = self.__class__.__name__
            test_method = self._testMethodName
            return "%s.%s.%s" % (test_module, test_class, test_method)
        else:
            return super(TestCase, self).id()


class FunctionalTestCase(unittest.TestCase):
    tags = ['functional']

    def assertTestRunOutputMatches(self, proc, stdout=None, stderr=None):
        cmd_stdout, cmd_stderr = None, None
        try:
            cmd_stdout, cmd_stderr = self._output[proc.pid]
        except AttributeError:
            self._output = {}
        except KeyError:
            pass
        if cmd_stdout is None:
            cmd_stdout, cmd_stderr = proc.communicate()
            self._output[proc.pid] = cmd_stdout, cmd_stderr
        testf = self.assertRegex if hasattr(self, 'assertRegex') \
            else self.assertRegexpMatches
        if stdout:
            testf(util.safe_decode(cmd_stdout), stdout)
        if stderr:
            testf(util.safe_decode(cmd_stderr), stderr)

    def runIn(self, testdir, *args, **kw):
        return run_nose2(*args, cwd=testdir, **kw)

    def runModuleAsMain(self, testmodule, *args):
        return run_module_as_main(testmodule, *args)


class _FakeEventBase(object):

    """Baseclass for fake :class:`~nose2.events.Event`\s."""

    def __init__(self):
        self.handled = False
        self.version = '0.1'
        self.metadata = {}


class FakeHandleFileEvent(_FakeEventBase):

    """Fake HandleFileEvent."""

    def __init__(self, name):
        super(FakeHandleFileEvent, self).__init__()

        self.loader = Stub()  # FIXME
        self.name = name
        self.path = os.path.split(name)[1]
        self.extraTests = []


class FakeStartTestEvent(_FakeEventBase):

    """Fake :class:`~nose2.events.StartTestEvent`."""

    def __init__(self, test):
        super(FakeStartTestEvent, self).__init__()
        self.test = test
        self.result = test.defaultTestResult()
        import time
        self.startTime = time.time()


class FakeLoadFromNameEvent(_FakeEventBase):

    """Fake :class:`~nose2.events.LoadFromNameEvent`."""

    def __init__(self, name):
        super(FakeLoadFromNameEvent, self).__init__()
        self.name = name


class FakeLoadFromNamesEvent(_FakeEventBase):

    """Fake :class:`~nose2.events.LoadFromNamesEvent`."""

    def __init__(self, names):
        super(FakeLoadFromNamesEvent, self).__init__()
        self.names = names


class FakeStartTestRunEvent(_FakeEventBase):

    """Fake :class:`~nose2.events.StartTestRunEvent`"""

    def __init__(self, runner=None, suite=None, result=None, startTime=None,
                 executeTests=None):
        super(FakeStartTestRunEvent, self).__init__()
        self.suite = suite
        self.runner = runner
        self.result = result
        self.startTime = startTime
        self.executeTests = executeTests


class Stub(object):

    """Stub object for use in tests"""

    def __getattr__(self, attr):
        return Stub()

    def __call__(self, *arg, **kw):
        return Stub()


def support_file(*path_parts):
    return os.path.abspath(os.path.join(SUPPORT, *path_parts))


def run_nose2(*nose2_args, **nose2_kwargs):
    if 'cwd' in nose2_kwargs:
        cwd = nose2_kwargs.pop('cwd')
        if not os.path.isabs(cwd):
            nose2_kwargs['cwd'] = support_file(cwd)
    return NotReallyAProc(nose2_args, **nose2_kwargs)


def run_module_as_main(test_module, *args):
    if not os.path.isabs(test_module):
        test_module = support_file(test_module)
    return subprocess.Popen([sys.executable, test_module] + list(args),
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)


class NotReallyAProc(object):

    def __init__(self, args, cwd=None, **kwargs):
        self.args = args
        self.chdir = cwd
        self.kwargs = kwargs
        self.result = None

    def __enter__(self):
        self._stdout = sys.__stdout__
        self._stderr = sys.__stderr__
        self.cwd = os.getcwd()
        if self.chdir:
            os.chdir(self.chdir)
        self.stdout = sys.stdout = sys.__stdout__ = six.StringIO()
        self.stderr = sys.stderr = sys.__stderr__ = six.StringIO()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        sys.stdout = sys.__stdout__ = self._stdout
        sys.stderr = sys.__stderr__ = self._stderr
        if self.chdir:
            os.chdir(self.cwd)
        return False

    def communicate(self):
        with self:
            try:
                self.result = discover(
                    argv=('nose2',) + self.args, exit=False,
                    **self.kwargs)
            except SystemExit as e:
                pass
            return self.stdout.getvalue(), self.stderr.getvalue()

    @property
    def pid(self):
        return id(self)

    def poll(self):
        if self.result is None:
            return 1
        return not self.result.result.wasSuccessful()


class RedirectStdStreams(object):

    """
    Context manager that replaces the stdin/stdout streams with :class:`StringIO`
    buffers.
    """

    def __init__(self):
        self.stdout = six.StringIO()
        self.stderr = six.StringIO()

    def __enter__(self):
        self.old_stdout, self.old_stderr = sys.stdout, sys.stderr
        self.old_stdout.flush()
        self.old_stderr.flush()
        sys.stdout, sys.stderr = self.stdout, self.stderr
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.stdout.flush()
        self.stderr.flush()
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr


# mock multprocessing Connection
class Conn(object):

    def __init__(self, items):
        self.items = items
        self.sent = []
        self.closed = False

    def recv(self):
        if self.closed:
            raise EOFError("closed")
        try:
            return self.items.pop(0)
        except:
            raise EOFError("EOF")

    def send(self, item):
        self.sent.append(item)

    def close(self):
        self.closed = True



"""
Collect and report test results.

This plugin implements the primary user interface for nose2. It
collects test outcomes and reports on them to the console, as well as
firing several hooks for other plugins to do their own reporting.

To see this report, nose2 MUST be run with the :option:`verbose` flag::

  nose2 --verbose

This plugin extends standard unittest console reporting slightly by
allowing custom report categories. To put events into a custom
reporting category, change the event.outcome to whatever you
want. Note, however, that customer categories are *not* treated as
errors or failures for the purposes of determining whether a test run
has succeeded.

Don't disable this plugin, unless you (a) have another one doing the
same job, or (b) really don't want any test results (and want all test
runs to ``exit(1)``).
"""
# This module contains some code copied from unittest2/runner.py and other
# code developed in reference to that module and others within unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html

import sys
import unittest

from nose2 import events, result, util

__unittest = True


class ResultReporter(events.Plugin):

    """Result plugin that implements standard unittest console reporting"""
    alwaysOn = True
    configSection = 'test-result'
    separator1 = '=' * 70
    separator2 = '-' * 70

    def __init__(self):
        self.testsRun = 0
        self.reportCategories = {'failures': [],
                                 'errors': [],
                                 'skipped': [],
                                 'expectedFailures': [],
                                 'unexpectedSuccesses': []}
        self.dontReport = set(['errors', 'failures', 'skipped', 'passed',
                               'expectedFailures', 'unexpectedSuccesses'])

        self.stream = util._WritelnDecorator(sys.stderr)
        self.descriptions = self.config.as_bool('descriptions', True)

    def startTest(self, event):
        """Handle startTest hook

        - prints test description if verbosity > 1
        """
        self.testsRun += 1
        self._reportStartTest(event)

    def testOutcome(self, event):
        """Handle testOutcome hook

        - records test outcome in reportCategories
        - prints test outcome label
        - fires reporting hooks (:func:`reportSuccess`, :func:`reportFailure`,
          etc)

        """
        if event.outcome == result.ERROR:
            self.reportCategories['errors'].append(event)
            self._reportError(event)
        elif event.outcome == result.FAIL:
            if not event.expected:
                self.reportCategories['failures'].append(event)
                self._reportFailure(event)
            else:
                self.reportCategories['expectedFailures'].append(event)
                self._reportExpectedFailure(event)
        elif event.outcome == result.SKIP:
            self.reportCategories['skipped'].append(event)
            self._reportSkip(event)
        elif event.outcome == result.PASS:
            if event.expected:
                self._reportSuccess(event)
            else:
                self.reportCategories['unexpectedSuccesses'].append(event)
                self._reportUnexpectedSuccess(event)
        else:
            # generic outcome handling
            self.reportCategories.setdefault(event.outcome, []).append(event)
            self._reportOtherOutcome(event)

    def afterTestRun(self, event):
        """Handle afterTestRun hook

        - prints error lists
        - prints summary
        - fires summary reporting hooks (:func:`beforeErrorList`,
          :func:`beforeSummaryReport`, etc)

        """
        self._reportSummary(event)

    def wasSuccessful(self, event):
        event.success = True
        for name, events in self.reportCategories.items():
            for e in events:
                if (e.outcome == result.ERROR or
                    (e.outcome == result.FAIL and not e.expected)):
                    event.success = False
                    break

    def _reportStartTest(self, event):
        evt = events.ReportTestEvent(event, self.stream)
        self.session.hooks.reportStartTest(evt)
        if evt.handled:
            return
        if self.session.verbosity > 1:
            # allow other plugins to override/spy on stream
            evt.stream.write(self._getDescription(event.test, errorList=False))
            evt.stream.write(' ... ')
            evt.stream.flush()

    def _reportError(self, event):
        self._report(event, 'reportError', 'E', 'ERROR')

    def _reportFailure(self, event):
        self._report(event, 'reportFailure', 'F', 'FAIL')

    def _reportSkip(self, event):
        self._report(event, 'reportSkip', 's', 'skipped %s' % event.reason)

    def _reportExpectedFailure(self, event):
        self._report(event, 'reportExpectedFailure', 'x', 'expected failure')

    def _reportUnexpectedSuccess(self, event):
        self._report(
            event, 'reportUnexpectedSuccess', 'u', 'unexpected success')

    def _reportOtherOutcome(self, event):
        self._report(event, 'reportOtherOutcome', '?', 'unknown outcome')

    def _reportSuccess(self, event):
        self._report(event, 'reportSuccess', '.', 'ok')

    def _reportSummary(self, event):
        # let others print something
        evt = events.ReportSummaryEvent(
            event, self.stream, self.reportCategories)
        self.session.hooks.beforeErrorList(evt)
        # allows other plugins to mess with report categories
        cats = evt.reportCategories
        errors = cats.get('errors', [])
        failures = cats.get('failures', [])
        # use evt.stream so plugins can replace/wrap/spy it
        evt.stream.writeln('')
        self._printErrorList('ERROR', errors, evt.stream)
        self._printErrorList('FAIL', failures, evt.stream)

        for flavour, events_ in cats.items():
            if flavour in self.dontReport:
                continue
            self._printErrorList(flavour.upper(), events_, evt.stream)
        self._printSummary(evt)

    def _printErrorList(self, flavour, events_, stream):
        for event in events_:
            desc = self._getDescription(event.test, errorList=True)
            err = self._getOutcomeDetail(event)
            stream.writeln(self.separator1)
            stream.writeln("%s: %s" % (flavour, desc))
            stream.writeln(self.separator2)
            stream.writeln("%s" % err)

    def _printSummary(self, reportEvent):
        self.session.hooks.beforeSummaryReport(reportEvent)

        stream = reportEvent.stream
        stream.writeln(self.separator2)
        run = self.testsRun
        msg = (
            "Ran %d test%s in %.3fs\n" %
            (run, run != 1 and "s" or "", reportEvent.stopTestEvent.timeTaken))
        stream.writeln(msg)

        infos = []
        extraInfos = []
        if reportEvent.stopTestEvent.result.wasSuccessful():
            stream.write("OK")
        else:
            stream.write("FAILED")

        failed = len(reportEvent.reportCategories.get('failures', []))
        errored = len(reportEvent.reportCategories.get('errors', []))
        skipped = len(reportEvent.reportCategories.get('skipped', []))
        expectedFails = len(
            reportEvent.reportCategories.get('expectedFailures', []))
        unexpectedSuccesses = len(
            reportEvent.reportCategories.get('unexpectedSuccesses', []))

        for flavour, results in reportEvent.reportCategories.items():
            if flavour in self.dontReport:
                continue
            count = len(results)
            if count:
                extraInfos.append("%s=%d" % (flavour, count))

        if failed:
            infos.append("failures=%d" % failed)
        if errored:
            infos.append("errors=%d" % errored)
        if skipped:
            infos.append("skipped=%d" % skipped)
        if expectedFails:
            infos.append("expected failures=%d" % expectedFails)
        if unexpectedSuccesses:
            infos.append("unexpected successes=%d" % unexpectedSuccesses)
        infos.extend(extraInfos)
        if infos:
            reportEvent.stream.writeln(" (%s)" % (", ".join(infos),))
        else:
            reportEvent.stream.writeln('')

        self.session.hooks.afterSummaryReport(reportEvent)

    def _getDescription(self, test, errorList):
        if not isinstance(test, unittest.TestCase):
            return test.__class__.__name__
        doc_first_line = test.shortDescription()
        if self.descriptions and doc_first_line:
            desc = '\n'.join((str(test), doc_first_line))
        else:
            desc = str(test)
        event = events.DescribeTestEvent(
            test, description=desc, errorList=errorList)
        self.session.hooks.describeTest(event)
        return event.description

    def _getOutcomeDetail(self, event):
        evt = events.OutcomeDetailEvent(event)
        result = self.session.hooks.outcomeDetail(evt)
        if evt.handled:
            return result
        exc_info = getattr(event, 'exc_info', None)
        test = getattr(event, 'test', None)
        if exc_info:
            detail = [util.exc_info_to_string(exc_info, test)]
        else:
            detail = []
        if evt.extraDetail:
            detail.extend(evt.extraDetail)
        try:
            return "\n".join(detail)
        except UnicodeDecodeError:
            return "\n".join(util.safe_decode(d) for d in detail)

    def _report(self, event, hook, shortLabel, longLabel):
        evt = events.ReportTestEvent(event, self.stream)
        getattr(self.session.hooks, hook)(evt)
        if evt.handled:
            return
        if self.session.verbosity > 1:
            # event I fired has stream, event I received has labels
            evt.stream.writeln(getattr(event, 'longLabel', None) or longLabel)
        elif self.session.verbosity:
            evt.stream.write(getattr(event, 'shortLabel', None) or shortLabel)
            evt.stream.flush()




class Test(object):
    def __init__(self):
        raise RuntimeError('Something bad happened but other tests should still be run!')
    
    def test(self):
        raise RuntimeError('Something bad happened but other tests should still be run! RUNNING')




try:
    import unittest2 as unittest
except ImportError:
    import unittest


class SomeTests(unittest.TestCase):
    tags = ['case']

    def test_ok(self):
        pass
    test_ok.tags = ['method', 'pass']
    test_ok.a = 0
    test_ok.b = 1

    def test_typeerr(self):
        raise TypeError("oops")
    test_typeerr.tags = ['method']

    def test_failed(self):
        print("Hello stdout")
        assert False, "I failed"
    test_failed.tags = ['method']

    def test_skippy(self):
        raise unittest.SkipTest("I wanted to skip")
    test_skippy.a = 1
    test_skippy.b = 1

    def test_gen_method(self):
        def check(x):
            assert x == 1
        check.b = 2
        yield check, 1
        yield check, 2
    test_gen_method.a = 1
    test_gen_method.b = 1

    def test_params_method(self, a):
        self.assertEqual(a, 1)
    test_params_method.paramList = (1, 2)
    test_params_method.a = 1


def test_func():
    assert 1 == 1
test_func.a = 1
test_func.b = 0
test_func.tags = ['func', 'pass']


def test_gen():
    def check(a, b):
        assert a == b
    check.tags = ['func']
    for i in range(0, 5):
        yield check, (i, i,)
test_gen.testGenerator = True
test_gen.tags = ['func']


def test_gen_nose_style():
    def check(a, b):
        assert a == b
    for i in range(0, 5):
        yield check, i, i


did_setup = False


def setup():
    global did_setup
    did_setup = True


def test_fixt():
    assert did_setup
test_fixt.setup = setup


def test_params_func(a):
    assert a == 1
test_params_func.paramList = (1, 2)
test_params_func.tags = ['func']


def test_params_func_multi_arg(a, b):
    assert a == b
test_params_func_multi_arg.paramList = ((1, 1), (1, 2), (2, 2))



"""
Use this plugin to activate coverage report.

To install this plugin, you need to activate ``coverage-plugin``
with extra requirements :

::

    $ pip install nose2[coverage-plugin]


Next, you can enable coverage reporting with :

::

    $ nose2 --with-coverage

Or with this lines in ``unittest.cfg`` :

::

    [coverage]
    always-on = True


"""
from nose2.events import Plugin


class Coverage(Plugin):
    configSection = 'coverage'
    commandLineSwitch = ('C', 'with-coverage', 'Turn on coverage reporting')

    def __init__(self):
        """Get our config and add our command line arguments."""
        self.conSource = self.config.as_list('coverage', [])
        self.conReport = self.config.as_list('coverage-report', [])
        self.conConfig = self.config.as_str('coverage-config', '').strip()

        group = self.session.pluginargs
        group.add_argument(
            '--coverage', action='append', default=[], metavar='PATH',
            dest='coverage_source',
            help='Measure coverage for filesystem path (multi-allowed)'
        )
        group.add_argument(
            '--coverage-report', action='append', default=[], metavar='TYPE',
            choices=['term', 'term-missing', 'annotate', 'html', 'xml'],
            dest='coverage_report',
            help='Generate selected reports, available types:'
                 ' term, term-missing, annotate, html, xml (multi-allowed)'
        )
        group.add_argument(
            '--coverage-config', action='store', default='', metavar='FILE',
            dest='coverage_config',
            help='Config file for coverage, default: .coveragerc'
        )
        self.covController = None

    def handleArgs(self, event):
        """Get our options in order command line, config file, hard coded."""

        self.covSource = (event.args.coverage_source or
                          self.conSource or ['.'])
        self.covReport = (event.args.coverage_report or
                          self.conReport or ['term'])
        self.covConfig = (event.args.coverage_config or
                          self.conConfig or '.coveragerc')

    def createTests(self, event):
        """Start coverage early to catch imported modules.

        Only called if active so, safe to just start without checking flags"""
        try:
            import cov_core
        except:
            print('Warning: you need to install [coverage-plugin] '
                  'extra requirements to use this plugin')
            return

        self.covController = cov_core.Central(self.covSource,
                                              self.covReport,
                                              self.covConfig)
        self.covController.start()

    def createdTestSuite(self, event):
        """Pause coverage collection until we begin running tests."""
        if self.covController:
            self.covController.cov.stop()

    def startTestRun(self, event):
        """Resume coverage collection before running tests."""
        if self.covController:
            self.covController.cov.start()

    def afterSummaryReport(self, event):
        """Only called if active so stop coverage and produce reports."""

        if self.covController:
            self.covController.finish()
            self.covController.summary(event.stream)



import re

from nose2.tests._common import FunctionalTestCase


class TestPrintHooksPlugin(FunctionalTestCase):

    def test_invocation_by_double_dash_option(self):
        proc = self.runIn(
            'scenario/no_tests',
            '--plugin=nose2.plugins.printhooks',
            '--print-hooks')
        match = re.compile("\n"
                           "handleArgs: "
                           "CommandLineArgsEvent\(handled=False, args=")
        self.assertTestRunOutputMatches(proc, stderr=match)
        self.assertEqual(proc.poll(), 0)

    def test_invocation_by_single_dash_option(self):
        proc = self.runIn(
            'scenario/no_tests',
            '--plugin=nose2.plugins.printhooks',
            '-P')
        match = re.compile("\n"
                           "handleArgs: "
                           "CommandLineArgsEvent\(handled=False, args=")
        self.assertTestRunOutputMatches(proc, stderr=match)
        self.assertEqual(proc.poll(), 0)

    def test_nested_hooks_are_indented(self):
        proc = self.runIn(
            'scenario/no_tests',
            '--plugin=nose2.plugins.printhooks',
            '--print-hooks')
        match = re.compile("\n  handleFile: ")
        self.assertTestRunOutputMatches(proc, stderr=match)
        self.assertEqual(proc.poll(), 0)



import sys

from nose2 import session
from nose2.tests._common import support_file, FunctionalTestCase


class SessionFunctionalTests(FunctionalTestCase):

    def setUp(self):
        self.s = session.Session()
        self.s.loadConfigFiles(support_file('cfg', 'a.cfg'),
                               support_file('cfg', 'b.cfg'))
        sys.path.insert(0, support_file('lib'))

    def test_session_can_load_config_files(self):
        assert self.s.config.has_section('a')
        assert self.s.config.has_section('b')

    def test_session_holds_plugin_config(self):
        plug_config = self.s.get('a')
        assert plug_config

    def test_session_can_load_plugins_from_modules(self):
        self.s.loadPlugins()
        assert self.s.plugins
        plug = self.s.plugins[0]
        self.assertEqual(plug.a, 1)



"""
Load tests from classes that are *not* :class:`unittest.TestCase` subclasses.

This plugin responds to :func:`loadTestsFromModule` by adding test
cases for test methods found in classes in the module that are *not*
sublcasses of :class:`unittest.TestCase`, but whose names (lowercased)
match the configured test method prefix.

Test class methods that are generators or have param lists are not
loaded here, but by the :class:`nose2.plugins.loader.generators.Generators` and
:class:`nose2.plugins.loader.parameters.Parameters` plugins.

This plugin also implements :func:`loadTestsFromName` to enable
loading tests from dotted class and method names passed on the command
line.

This plugin makes two additional plugin hooks available for other
test loaders to use:

.. function :: loadTestsFromTestClass(self, event)

   :param event: A :class:`LoadFromTestClassEvent` instance

   Plugins can use this hook to load tests from a class that is not a
   :class:`unittest.TestCase` subclass. To prevent other plugins from
   loading tests from the test class, set ``event.handled`` to ``True`` and
   return a test suite. Plugins can also append tests to
   ``event.extraTests``. Ususally, that's what you want, since
   it allows other plugins to load their tests from the test
   case as well.

.. function :: getTestMethodNames(self, event)

   :param event: A :class:`GetTestMethodNamesEvent` instance

   Plugins can use this hook to limit or extend the list of test case
   names that will be loaded from a class that is not a
   :class:`unittest.TestCase` subclass by the standard nose2 test
   loader plugins (and other plugins that respect the results of the
   hook). To force a specific list of names, set ``event.handled`` to
   ``True`` and return a list: this exact list will be the only test case
   names loaded from the test case. Plugins can also extend the list
   of names by appending test names to ``event.extraNames``, and
   exclude names by appending test names to ``event.excludedNames``.

About Test Classes
------------------

Test classes are classes that look test-like but are not subclasses of
:class:`unittest.TestCase`. Test classes support all of the same test
types and fixtures as test cases.

To "look test-like" a class must have a name that, lowercased, matches
the configured test method prefix -- "test" by default. Test classes
must also be able to be instantiated without arguments.

What are they useful for? Mostly the case where a test class can't for
some reason subclass :class:`unittest.TestCase`. Otherwise, test class
tests and test cases are functionally equivalent in nose2, and test
cases have broader support and all of those helpful *assert\** methods
-- so when in doubt, you should use a :class:`unittest.TestCase`.

Here's an example of a test class::

  class TestSomething(object):

      def test(self):
          assert self.something(), "Something failed!"

"""

import unittest
import sys

from nose2 import events, util
from nose2.compat import unittest as ut2

__unittest = True


class TestClassLoader(events.Plugin):

    """Loader plugin that loads test functions"""
    alwaysOn = True
    configSection = 'test-classes'

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)

    def register(self):
        """Install extra hooks

        Adds the new plugin hooks:

        - loadTestsFromTestClass
        - getTestMethodNames

        """
        super(TestClassLoader, self).register()
        self.addMethods('loadTestsFromTestClass', 'getTestMethodNames')

    def loadTestsFromModule(self, event):
        """Load test classes from event.module"""
        module = event.module
        for name in dir(module):
            obj = getattr(module, name)
            if (isinstance(obj, type) and
                not issubclass(obj, unittest.TestCase) and
                not issubclass(obj, unittest.TestSuite) and
                name.lower().startswith(self.session.testMethodPrefix)):
                event.extraTests.append(
                    self._loadTestsFromTestClass(event, obj))

    def loadTestsFromName(self, event):
        """Load tests from event.name if it names a test class/method"""
        name = event.name
        module = event.module
        try:
            result = util.test_from_name(name, module)
        except (AttributeError, ImportError) as e:
            event.handled = True
            return event.loader.failedLoadTests(name, e)
        if result is None:
            return
        parent, obj, name, index = result
        if isinstance(obj, type) and not issubclass(obj, unittest.TestCase):
            # name is a test case class
            event.extraTests.append(self._loadTestsFromTestClass(event, obj))
        elif (isinstance(parent, type) and
              not issubclass(parent, unittest.TestCase) and
              not util.isgenerator(obj) and
              not hasattr(obj, 'paramList')):
            # name is a single test method
            event.extraTests.append(
                util.transplant_class(
                    MethodTestCase(parent), parent.__module__)(obj.__name__))

    def _loadTestsFromTestClass(self, event, cls):
        # ... fire event for others to load from
        evt = LoadFromTestClassEvent(event.loader, cls)
        result = self.session.hooks.loadTestsFromTestClass(evt)
        if evt.handled:
            loaded_suite = result or event.loader.suiteClass()
        else:
            names = self._getTestMethodNames(event, cls)
            try:
                loaded_suite = event.loader.suiteClass(
                    [util.transplant_class(
                     MethodTestCase(cls), cls.__module__)(name)
                        for name in names])
            except:
                _, ev, _ = sys.exc_info()
                return event.loader.suiteClass(
                    event.loader.failedLoadTests(cls.__name__, ev))
        if evt.extraTests:
            loaded_suite.addTests(evt.extraTests)
        # ... add extra tests
        return loaded_suite

    def _getTestMethodNames(self, event, cls):
        # ... give others a chance to modify list
        excluded = set()

        def isTestMethod(attrname, cls=cls, excluded=excluded):
            # FIXME allow plugs to change prefix
            prefix = self.session.testMethodPrefix
            return (
                attrname.startswith(prefix) and
                hasattr(getattr(cls, attrname), '__call__') and
                attrname not in excluded
            )
        evt = GetTestMethodNamesEvent(event.loader, cls, isTestMethod)
        result = self.session.hooks.getTestMethodNames(evt)
        if evt.handled:
            test_names = result or []
        else:
            excluded.update(evt.excludedNames)

            test_names = [entry for entry in dir(cls)
                          if isTestMethod(entry)]

        if event.loader.sortTestMethodsUsing:
            test_names.sort(key=event.loader.sortTestMethodsUsing)
        return test_names


# to prevent unit2 discover from running this as a test, need to
# hide it inside of a factory func. ugly!
def MethodTestCase(cls):
    class _MethodTestCase(ut2.TestCase):

        def __init__(self, method):
            self.method = method
            self._name = "%s.%s.%s" % (cls.__module__, cls.__name__, method)
            self.obj = cls()
            ut2.TestCase.__init__(self, 'runTest')

        @classmethod
        def setUpClass(klass):
            if hasattr(cls, 'setUpClass'):
                cls.setUpClass()

        @classmethod
        def tearDownClass(klass):
            if hasattr(cls, 'tearDownClass'):
                cls.tearDownClass()

        def setUp(self):
            if hasattr(self.obj, 'setUp'):
                self.obj.setUp()

        def tearDown(self):
            if hasattr(self.obj, 'tearDown'):
                self.obj.tearDown()

        def __repr__(self):
            return self._name
        id = __str__ = __repr__

        def runTest(self):
            getattr(self.obj, self.method)()

    return _MethodTestCase

#
# Event classes
#


class LoadFromTestClassEvent(events.LoadFromTestCaseEvent):

    """Bare subclass of :class:`nose2.events.LoadFromTestCaseEvent`"""


class GetTestMethodNamesEvent(events.GetTestCaseNamesEvent):

    """Bare subclass of :class:`nose2.events.GetTestCaseNamesEvent`"""



import re

from nose2.tests._common import FunctionalTestCase


class LogCaptureFunctionalTest(FunctionalTestCase):

    def test_package_in_lib(self):
        match = re.compile('>> begin captured logging <<')
        self.assertTestRunOutputMatches(
            self.runIn('scenario/package_in_lib', '--log-capture'),
            stderr=match)

    def test_logging_keeps_copies_of_mutable_objects(self):
        proc = self.runIn('scenario/logging',
                          '-v',
                          '--log-capture',
                          'logging_keeps_copies_of_mutable_objects')
        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test in')
        self.assertTestRunOutputMatches(proc, stderr='FAILED')
        self.assertTestRunOutputMatches(proc, stderr='foo: {}')



"""
Load tests from test functions in modules.

This plugin responds to :func:`loadTestsFromModule` by adding test
cases for all test functions in the module to ``event.extraTests``. It
uses ``session.testMethodPrefix`` to find test functions.

Functions that are generators, have param lists, or take arguments
are not collected.

This plugin also implements :func:`loadTestsFromName` to enable loading
tests from dotted function names passed on the command line.

Fixtures
--------

Test functions can specify setup and teardown fixtures as attributes on the
function, for example:

.. code :: python

   x = 0

   def test():
       assert x

   def setup():
       global x
       x = 1

   def teardown():
       global x
       x = 1

   test.setup = setup
   test.teardown = teardown

The setup attribute may be named ``setup``, ``setUp`` or ``setUpFunc``. The
teardown attribute may be named ``teardown``, 'tearDown`` or ``tearDownFunc``.

Other attributes
----------------

The other significant attribute that may be set on a test function is
``paramList``. When ``paramList`` is set, the function will be collected
by the :doc:`parameterized test loader <parameters>`. The easiest way
to set ``paramList`` is with the :func:`nose2.tools.params` decorator.

"""
# This module contains some code copied from unittest2/ and other code
# developed in reference to unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html


import inspect
import types

from nose2 import util
from nose2.events import Plugin
from nose2.compat import unittest


__unittest = True


class Functions(Plugin):

    """Loader plugin that loads test functions"""
    alwaysOn = True
    configSection = 'functions'

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)

    def loadTestsFromName(self, event):
        """Load test if event.name is the name of a test function"""
        name = event.name
        module = event.module
        try:
            result = util.test_from_name(name, module)
        except (AttributeError, ImportError) as e:
            event.handled = True
            return event.loader.failedLoadTests(name, e)
        if result is None:
            return

        parent, obj, name, index = result
        if (isinstance(obj, types.FunctionType) and not
            util.isgenerator(obj) and not
            hasattr(obj, 'paramList') and not
            inspect.getargspec(obj).args):
            suite = event.loader.suiteClass()
            suite.addTests(self._createTests(obj))
            event.handled = True
            return suite

    def loadTestsFromModule(self, event):
        """Load test functions from event.module"""
        module = event.module

        def is_test(obj):
            if not obj.__name__.startswith(self.session.testMethodPrefix):
                return False
            if inspect.getargspec(obj).args:
                return False
            return True

        tests = []
        for name in dir(module):
            obj = getattr(module, name)
            if isinstance(obj, types.FunctionType) and is_test(obj):
                tests.extend(self._createTests(obj))
        event.extraTests.extend(tests)

    def _createTests(self, obj):
        if not hasattr(obj, 'setUp'):
            if hasattr(obj, 'setup'):
                obj.setUp = obj.setup
            elif hasattr(obj, 'setUpFunc'):
                obj.setUp = obj.setUpFunc
        if not hasattr(obj, 'tearDown'):
            if hasattr(obj, 'teardown'):
                obj.tearDown = obj.teardown
            elif hasattr(obj, 'tearDownFunc'):
                obj.tearDown = obj.tearDownFunc

        tests = []
        args = {}
        setUp = getattr(obj, 'setUp', None)
        tearDown = getattr(obj, 'tearDown', None)
        if setUp is not None:
            args['setUp'] = setUp
        if tearDown is not None:
            args['tearDown'] = tearDown

        paramList = getattr(obj, 'paramList', None)
        isGenerator = util.isgenerator(obj)
        if paramList is not None or isGenerator:
            return tests
        else:
            case = util.transplant_class(
                unittest.FunctionTestCase, obj.__module__)(obj, **args)
            tests.append(case)
        return tests



import os
import sys

from nose2.tests._common import TestCase
from nose2 import util


class UtilTests(TestCase):
    _RUN_IN_TEMP = True

    def test_ensure_importable(self):
        test_dir = os.path.join(self._work_dir, 'test_dir')
        # Make sure test data is suitable for the test
        self.assertNotIn(test_dir, sys.path)
        util.ensure_importable(test_dir)
        self.assertEqual(test_dir, sys.path[0])



from nose2 import config
from nose2.compat import unittest


class TestConfig(unittest.TestCase):

    def setUp(self):
        self.conf = config.Config([
            ('a', ' 1 '), ('b', '  x\n  y '), ('c', '0'),
            ('d', '123')])

    def test_as_int(self):
        self.assertEqual(self.conf.as_int('a'), 1)

    def test_as_str(self):
        self.assertEqual(self.conf.as_str('a'), '1')
        self.assertEqual(self.conf.as_str('b'), 'x\n  y')
        self.assertEqual(self.conf.as_str('missing', 'default'), 'default')

    def test_as_bool(self):
        self.assertEqual(self.conf.as_bool('a'), True)
        self.assertEqual(self.conf.as_bool('c'), False)

    def test_as_float(self):
        self.assertAlmostEqual(self.conf.as_float('a'), 1.0)

    def test_as_list(self):
        self.assertEqual(self.conf.as_list('b'), ['x', 'y'])
        self.assertEqual(self.conf.as_list('a'), ['1'])
        self.assertEqual(self.conf.as_list('d'), ['123'])



from nose2.tests._common import FakeStartTestRunEvent, TestCase
from nose2.plugins import collect
from nose2 import session


class TestCollectOnly(TestCase):
    tags = ['unit']

    def setUp(self):
        self.session = session.Session()
        self.plugin = collect.CollectOnly(session=self.session)

    def test_startTestRun_sets_executeTests(self):
        event = FakeStartTestRunEvent()
        self.plugin.startTestRun(event)
        self.assertEqual(event.executeTests, self.plugin.collectTests)



from nose2.tools import such

with such.A('test scenario with errors') as it:

    @it.has_setup
    def set_value():
        it.value = 1

    @it.should('check that value == 1')
    def test_passes(case):
        case.assertEqual(it.value, 1)

    @it.should('check that value == 2 and fail')
    def test_fails(case):
        case.assertEqual(it.value, 2)

    @it.should('check for an attribute that does not exist and raise an error')
    def test_err(case):
        case.assertEqual(it.mulch, 'pine')


it.createTests(globals())



from nose2 import result, session
from nose2.tests._common import TestCase


class TestPluggableTestResult(TestCase):

    def setUp(self):
        self.session = session.Session()
        self.result = result.PluggableTestResult(self.session)

    def test_skip_reason_not_discarded(self):
        class Test(TestCase):

            def test(self):
                pass
        plugin = FakePlugin()
        self.session.hooks.register('testOutcome', plugin)
        self.result.addSkip(Test('test'), 'because')
        self.assertEqual(plugin.reason, 'because')


class FakePlugin(object):

    def testOutcome(self, event):
        self.reason = event.reason



"""
Allow easy test selection with test ids.

Assigns (and, in verbose mode, prints) a sequential test id for each
test executed. Ids can be fed back in as test names, and this plugin
will translate them back to full test names. Saves typing!

This plugin implements :func:`reportStartTest`,
:func:`loadTestsFromName`, :func:`loadTestsFromNames` and
:func:`stopTest`.

"""
import os
import pickle
import re

from nose2.events import Plugin
from nose2 import util


__unittest = True


class TestId(Plugin):

    """Allow easy test select with ids"""

    configSection = 'testid'
    commandLineSwitch = ('I', 'with-id', 'Add test ids to output')
    idpat = re.compile(r'(\d+)')

    def __init__(self):
        self.idfile = self.config.as_str('id-file', '.noseids')
        self.ids = {}
        self.tests = {}
        if not os.path.isabs(self.idfile):
            # FIXME expand-user?
            self.idfile = os.path.join(os.getcwd(), self.idfile)
        self.id = 0
        self._loaded = False

    def nextId(self):
        """Increment ID and return it."""
        self.id += 1
        return self.id

    def reportStartTest(self, event):
        """Record and possibly output test id"""
        testid = util.test_name(event.testEvent.test)
        if testid not in self.tests:
            id_ = self.nextId()
            self.ids[id_] = testid
            self.tests[testid] = id_
        else:
            id_ = self.tests[testid]
        event.metadata['testid'] = id_
        if self.session.verbosity > 1:
            event.stream.write('#%s ' % id_)

    def loadTestsFromName(self, event):
        """Load tests from a name that is an id

        If the name is a number, it might be an ID assigned by us. If we can
        find a test to which we have assigned that ID, event.name is changed to
        the test's real ID. In this way, tests can be referred to via sequential
        numbers.
        """
        testid = self._testNameFromId(event.name)
        if testid is not None:
            event.name = testid

    def loadTestsFromNames(self, event):
        """Translate test ids into test names"""
        for i, name in enumerate(event.names[:]):
            testid = self._testNameFromId(name)
            if testid is not None:
                event.names[i] = testid

    def stopTestRun(self, event):
        """Write testids file"""
        with open(self.idfile, 'wb') as fh:
            pickle.dump({'ids': self.ids, 'tests': self.tests}, fh)

    def loadIds(self):
        """Load previously pickled 'ids' and 'tests' attributes."""
        if self._loaded:
            return

        try:
            with open(self.idfile, 'rb') as fh:
                data = pickle.load(fh)
        except EnvironmentError:
            self._loaded = True
            return

        if 'ids' in data:
            self.ids = data['ids']
        if 'tests' in data:
            self.tests = data['tests']
        self.id = max(self.ids.keys())
        self._loaded = True

    def _testNameFromId(self, name):
        """Try to translate one of our IDs to real test ID."""
        m = self.idpat.match(name)
        if m is None:
            return None

        id_ = int(m.groups()[0])

        self.loadIds()
        # Translate to test's real ID
        try:
            return self.ids[id_]
        except KeyError:
            return None



from nose2.compat import unittest

STATE = {}


class L1(object):

    @classmethod
    def setUp(cls):
        STATE['L1'] = 'setup'

    @classmethod
    def tearDown(cls):
        del STATE['L1']


class L2(object):

    @classmethod
    def setUp(cls):
        STATE['L2'] = 'setup'

    @classmethod
    def tearDown(cls):
        del STATE['L2']


class LayerAndAttributesA(unittest.TestCase):
    layer = L1
    a = 1

    def test(self):
        self.assertEqual(STATE.get('L1'), 'setup')


class LayerAndAttributesB(unittest.TestCase):
    layer = L2
    b = 1

    def test(self):
        self.assertEqual(STATE.get('L2'), 'setup')



import sys

from nose2 import session
from nose2.compat import unittest
from nose2.plugins.mp import MultiProcess, procserver
from nose2.plugins import buffer
from nose2.plugins.loader import discovery, testcases
from nose2.tests._common import FunctionalTestCase, support_file, Conn
from six.moves import queue
import multiprocessing
import threading
import time
from multiprocessing import connection


class TestMpPlugin(FunctionalTestCase):

    def setUp(self):
        super(TestMpPlugin, self).setUp()
        self.session = session.Session()
        self.plugin = MultiProcess(session=self.session)

    def test_flatten_without_fixtures(self):
        sys.path.append(support_file('scenario/slow'))
        import test_slow as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.TestSlow('test_ok'))
        suite.addTest(mod.TestSlow('test_fail'))
        suite.addTest(mod.TestSlow('test_err'))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(len(flat), 3)

    def test_flatten_nested_suites(self):
        sys.path.append(support_file('scenario/slow'))
        import test_slow as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.TestSlow('test_ok'))
        suite.addTest(mod.TestSlow('test_fail'))
        suite.addTest(mod.TestSlow('test_err'))

        suite2 = unittest.TestSuite()
        suite2.addTest(suite)

        flat = list(self.plugin._flatten(suite2))
        self.assertEqual(len(flat), 3)

    def test_flatten_respects_module_fixtures(self):
        sys.path.append(support_file('scenario/module_fixtures'))
        import test_mf_testcase as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.Test('test_1'))
        suite.addTest(mod.Test('test_2'))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(flat, ['test_mf_testcase'])

    def test_flatten_respects_class_fixtures(self):
        sys.path.append(support_file('scenario/class_fixtures'))
        import test_cf_testcase as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.Test('test_1'))
        suite.addTest(mod.Test('test_2'))
        suite.addTest(mod.Test2('test_1'))
        suite.addTest(mod.Test2('test_2'))
        suite.addTest(mod.Test3('test_3'))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(flat, ['test_cf_testcase.Test2.test_1',
                                'test_cf_testcase.Test2.test_2',
                                'test_cf_testcase.Test',
                                'test_cf_testcase.Test3',
                                ])

    def test_conn_prep(self):
        self.plugin.bind_host = None
        (parent_conn, child_conn) = self.plugin._prepConns()
        (parent_pipe, child_pipe) = multiprocessing.Pipe()
        self.assertIsInstance(parent_conn, type(parent_pipe))
        self.assertIsInstance(child_conn, type(child_pipe))

        self.plugin.bind_host = "127.0.0.1"
        self.plugin.bind_port = 0
        (parent_conn, child_conn) = self.plugin._prepConns()
        self.assertIsInstance(parent_conn, connection.Listener)
        self.assertIsInstance(child_conn, tuple)
        self.assertEqual(parent_conn.address, child_conn[:2])

    def test_conn_accept(self):
        (parent_conn, child_conn) = multiprocessing.Pipe()
        self.assertEqual(self.plugin._acceptConns(parent_conn), parent_conn)

        listener = connection.Listener(('127.0.0.1', 0))
        with self.assertRaises(RuntimeError):
            self.plugin._acceptConns(listener)

        def fake_client(address):
            client = connection.Client(address)
            time.sleep(10)
            client.close()

        t = threading.Thread(target=fake_client, args=(listener.address,))
        t.start()
        conn = self.plugin._acceptConns(listener)
        self.assertTrue(hasattr(conn, "send"))
        self.assertTrue(hasattr(conn, "recv"))


class TestProcserver(FunctionalTestCase):

    def setUp(self):
        super(TestProcserver, self).setUp()
        self.session = session.Session()

    def test_dispatch_tests_receive_events(self):
        ssn = {
            'config': self.session.config,
            'verbosity': 1,
            'startDir': support_file('scenario/tests_in_package'),
            'topLevelDir': support_file('scenario/tests_in_package'),
            'logLevel': 100,
            'pluginClasses': [discovery.DiscoveryLoader,
                              testcases.TestCaseLoader,
                              buffer.OutputBufferPlugin]

        }
        conn = Conn(['pkg1.test.test_things.SomeTests.test_ok',
                     'pkg1.test.test_things.SomeTests.test_failed'])
        procserver(ssn, conn)

        # check conn calls
        expect = [('pkg1.test.test_things.SomeTests.test_ok',
                   [('startTest', {}),
                    ('setTestOutcome', {'outcome': 'passed'}),
                    ('testOutcome', {'outcome': 'passed'}),
                    ('stopTest', {})]
                   ),
                  ('pkg1.test.test_things.SomeTests.test_failed',
                   [('startTest', {}),
                    ('setTestOutcome', {
                     'outcome': 'failed',
                     'expected': False,
                     'metadata': {'stdout': 'Hello stdout\n'}}),
                    ('testOutcome', {
                     'outcome': 'failed',
                     'expected': False,
                     'metadata': {'stdout': 'Hello stdout\n'}}),
                    ('stopTest', {})]
                   ),
                  ]
        for val in conn.sent:
            if val is None:
                break
            test, events = val
            exp_test, exp_events = expect.pop(0)
            self.assertEqual(test, exp_test)
            for method, event in events:
                exp_meth, exp_attr = exp_events.pop(0)
                self.assertEqual(method, exp_meth)
                for attr, val in exp_attr.items():
                    self.assertEqual(getattr(event, attr), val)


class MPPluginTestRuns(FunctionalTestCase):

    def test_tests_in_package(self):
        proc = self.runIn(
            'scenario/tests_in_package',
            '-v',
            '--plugin=nose2.plugins.mp',
            '-N=2')
        self.assertTestRunOutputMatches(proc, stderr='Ran 25 tests')
        self.assertEqual(proc.poll(), 1)

    def test_package_in_lib(self):
        proc = self.runIn(
            'scenario/package_in_lib',
            '-v',
            '--plugin=nose2.plugins.mp',
            '-N=2')
        self.assertTestRunOutputMatches(proc, stderr='Ran 3 tests')
        self.assertEqual(proc.poll(), 1)

    def test_module_fixtures(self):
        proc = self.runIn(
            'scenario/module_fixtures',
            '-v',
            '--plugin=nose2.plugins.mp',
            '-N=2')
        self.assertTestRunOutputMatches(proc, stderr='Ran 5 tests')
        self.assertEqual(proc.poll(), 0)

    def test_class_fixtures(self):
        proc = self.runIn(
            'scenario/class_fixtures',
            '-v',
            '--plugin=nose2.plugins.mp',
            '-N=2')
        self.assertTestRunOutputMatches(proc, stderr='Ran 7 tests')
        self.assertEqual(proc.poll(), 0)

    def test_large_number_of_tests_stresstest(self):
        proc = self.runIn(
            'scenario/many_tests',
            '-v',
            '--plugin=nose2.plugins.mp',
            '--plugin=nose2.plugins.loader.generators',
            '-N=1')
        self.assertTestRunOutputMatches(proc, stderr='Ran 600 tests')
        self.assertEqual(proc.poll(), 0)

    def test_socket_stresstest(self):
        proc = self.runIn(
            'scenario/many_tests_socket',
            '-v',
            '-c scenario/many_test_socket/nose2.cfg',
            '--plugin=nose2.plugins.mp',
            '--plugin=nose2.plugins.loader.generators',
            '-N=1')
        self.assertTestRunOutputMatches(proc, stderr='Ran 600 tests')
        self.assertEqual(proc.poll(), 0)

    def test_too_many_procs(self):
        # Just need to run the mp plugin with less tests than
        # processes.
        proc = self.runModuleAsMain('scenario/one_test/tests.py',
                                    '--log-level=debug',
                                    '--plugin=nose2.plugins.mp',
                                    '-N=2')
        ret_vals = queue.Queue()

        def save_return():
            """
            Popen.communciate() blocks.  Use a thread-safe queue
            to return any exceptions.  Ideally, this completes
            and returns None.
            """
            try:
                self.assertTestRunOutputMatches(proc,
                                                stderr='Ran 1 test')
                self.assertEqual(proc.poll(), 0)
                ret_vals.put(None)
            except Exception as exc:
                ret_vals.put(exc)
        thread = threading.Thread(target=save_return)
        thread.start()

        # 1 minute should be more than sufficent for this
        # little test case.
        try:
            exc = ret_vals.get(True, 60)
        except queue.Empty:
            exc = "MP Test timed out"
            proc.kill()
        self.assertIsNone(exc, str(exc))




import os

from nose2.tests._common import FunctionalTestCase
from nose2.tests._common import TestCase
from nose2.tests._common import support_file

from xml.etree import ElementTree as ET


class JunitXmlPluginFunctionalTest(FunctionalTestCase, TestCase):
    _RUN_IN_TEMP = True

    def run_with_junitxml_loaded(self, scenario, *args):
        work_dir = os.getcwd()
        test_dir = support_file(*scenario)
        junit_report = os.path.join(work_dir, 'nose2-junit.xml')
        config = os.path.join(test_dir, 'unittest.cfg')
        config_args = ()
        if os.path.exists(junit_report):
            os.remove(junit_report)
        if os.path.exists(config):
            config_args = ('-c', config)
        proc = self.runIn(work_dir,
                          '-s%s' % test_dir,
                          '--plugin=nose2.plugins.junitxml',
                          '-v',
                          *(config_args + args))
        return junit_report, proc

    def test_invocation_by_double_dash_option(self):
        junit_report, proc = self.run_with_junitxml_loaded(
            ('scenario', 'junitxml', 'happyday'),
            '--junit-xml')

        self.assertTestRunOutputMatches(
            proc, stderr='test \(test_junitxml_happyday.Test\) ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Ran 1 test')
        self.assertEqual(proc.poll(), 0)

        self.assertTrue(os.path.isfile(junit_report),
                        "junitxml report wasn't found in working directory. "
                        "Searched for " + junit_report)

    def test_invocation_by_single_dash_option(self):
        junit_report, proc = self.run_with_junitxml_loaded(
            ('scenario', 'junitxml', 'happyday'),
            '-X')

        self.assertTestRunOutputMatches(
            proc, stderr='test \(test_junitxml_happyday.Test\) ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Ran 1 test')
        self.assertEqual(proc.poll(), 0)

        self.assertTrue(os.path.isfile(junit_report),
                        "junitxml report wasn't found in working directory. "
                        "Searched for " + junit_report)

    def test_no_report_written_if_loaded_but_not_invoked(self):
        junit_report, proc = self.run_with_junitxml_loaded(
            ('scenario', 'junitxml', 'happyday'))

        self.assertTestRunOutputMatches(
            proc, stderr='test \(test_junitxml_happyday.Test\) ... ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Ran 1 test')
        self.assertEqual(proc.poll(), 0)

        self.assertFalse(os.path.isfile(junit_report),
                         "junitxml report was found in working directory. "
                         "Report file: " + junit_report)

    def test_report_location_should_be_resilent_to_chdir_in_tests(self):
        junit_report, proc = self.run_with_junitxml_loaded(
            ('scenario', 'junitxml', 'chdir'), '--junit-xml')

        self.assertTestRunOutputMatches(
            proc,
            stderr='test_chdir \(test_junitxml_chdir.Test\) \.* ok')
        self.assertTestRunOutputMatches(
            proc, stderr='Ran 1 test')
        self.assertEqual(proc.poll(), 0)

        self.assertTrue(os.path.isfile(junit_report),
                        "junitxml report wasn't found in working directory. "
                        "Searched for " + junit_report)

    def test_report_includes_properties(self):
        scenario = ('scenario', 'junitxml', 'with_properties')
        work_dir = os.getcwd()
        with open(os.path.join(work_dir, 'properties.json'), 'w') as fh:
            fh.write('{"PROPERTY_NAME":"PROPERTY_VALUE"}')
        junit_report, proc = self.run_with_junitxml_loaded(
            ('scenario', 'junitxml', 'with_properties'), '--junit-xml')
        self.assertTestRunOutputMatches(
            proc,
            stderr='test \(test_junitxml_with_properties.Test\) \.* ok')
        self.assertEqual(proc.poll(), 0)

        with open(junit_report, 'r') as fh:
            tree = ET.parse(fh).getroot()
        self.assertEqual(len(tree.findall('properties')), 1)
        prop = tree.find('properties').find('property')
        assert 'name' in prop.attrib
        assert 'value' in prop.attrib
        self.assertEqual(prop.get('name'), 'PROPERTY_NAME')
        self.assertEqual(prop.get('value'), 'PROPERTY_VALUE')


class JunitXmlPluginFunctionalFailureTest(FunctionalTestCase, TestCase):
    def test_failure_to_write_report(self):
        proc = self.runIn('scenario/junitxml/fail_to_write',
                          '--plugin=nose2.plugins.junitxml',
                          '-v',
                          '--junit-xml')
        self.assertEqual(proc.poll(), 1)

        self.assertTestRunOutputMatches(
            proc,
            stderr='test \(test_junitxml_fail_to_write.Test\) \.* ok')

        filename_for_regex = os.path.abspath('/does/not/exist.xml')
        filename_for_regex = filename_for_regex.replace('\\', r'\\\\')
        self.assertTestRunOutputMatches(
            proc, stderr="Internal Error: runTests aborted: \[Errno 2\] "
                         "JUnitXML: Parent folder does not exist for file: "
                         "\'%s'" % filename_for_regex)

    def test_failure_to_read_missing_properties(self):
        proc = self.runIn('scenario/junitxml/missing_properties',
                          '--plugin=nose2.plugins.junitxml',
                          '-v',
                          '--junit-xml')
        self.assertEqual(proc.poll(), 1)

        self.assertTestRunOutputMatches(
            proc,
            stderr='test \(test_junitxml_missing_properties.Test\) \.* ok')

        filename_for_regex = os.path.join('missing_properties',
                                          'properties.json')
        filename_for_regex = filename_for_regex.replace('\\', r'\\\\')
        self.assertTestRunOutputMatches(
            proc, stderr="Internal Error: runTests aborted: \[Errno 2\] "
                         "JUnitXML: Properties file does not exist: "
                         "'.*%s'" % filename_for_regex)


    def test_failure_to_read_empty_properties(self):
        proc = self.runIn('scenario/junitxml/empty_properties',
                          '--plugin=nose2.plugins.junitxml',
                          '-v',
                          '--junit-xml')
        self.assertEqual(proc.poll(), 1)

        self.assertTestRunOutputMatches(
            proc,
            stderr='test \(test_junitxml_empty_properties.Test\) \.* ok')

        filename_for_regex = os.path.join('empty_properties',
                                          'properties.json')
        filename_for_regex = filename_for_regex.replace('\\', r'\\')
        self.assertTestRunOutputMatches(
            proc, stderr="Internal Error: runTests aborted: "
                         "JUnitXML: could not decode file: "
                         "'.*%s'" % filename_for_regex)



import os
import sys

NAME = 'nose2'
VERSION = '0.6.4'
PACKAGES = ['nose2', 'nose2.plugins', 'nose2.plugins.loader',
            'nose2.tests', 'nose2.tests.functional', 'nose2.tests.unit',
            'nose2.tools', 'nose2.backports']
SCRIPTS = ['bin/nose2']
DESCRIPTION = 'nose2 is the next generation of nicer testing for Python'
URL = 'https://github.com/nose-devs/nose2'
LONG_DESCRIPTION = open(
    os.path.join(os.path.dirname(__file__), 'README.rst')).read()

CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: Implementation :: CPython',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Operating System :: OS Independent',
    'Topic :: Software Development :: Libraries',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Software Development :: Testing',
]

AUTHOR = 'Jason Pellerin'
AUTHOR_EMAIL = 'jpellerin+nose@gmail.com'
KEYWORDS = ['unittest', 'testing', 'tests']

params = dict(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=PACKAGES,
    scripts=SCRIPTS,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    classifiers=CLASSIFIERS,
    keywords=KEYWORDS,
    extras_require={
        'coverage_plugin': ["cov-core>=1.12"],
    }
)


py_version = sys.version[:3]

SCRIPT1 = 'nose2'
SCRIPT2 = 'nose2-%s' % (py_version,)

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
else:

    REQS = ['six>=1.1']
    if sys.version_info < (2, 7):
        REQS.extend(['unittest2>=0.5.1,<0.6', 'argparse>=1.2.1,<1.3'])

    params['entry_points'] = {
        'console_scripts': [
            '%s = nose2:discover' % SCRIPT1,
            '%s = nose2:discover' % SCRIPT2,
        ],
    }
    params['install_requires'] = REQS
    params['test_suite'] = 'nose2.compat.unittest.collector'

setup(**params)



from nose2.tests._common import FunctionalTestCase


class TestDunderTestPlugin(FunctionalTestCase):
    def test_dunder(self):
        proc = self.runIn(
            'scenario/dundertest_attribute',
            '-v')
        self.assertTestRunOutputMatches(proc, stderr='Ran 0 tests')



THINGS = []


def setUpModule():
    THINGS.append(1)


def tearDownModule():
    while THINGS:
        THINGS.pop()


def check(_):
    assert THINGS, "setup didn't run I think"


def test():
    yield check, 1



"""
Output test reports in junit-xml format.

This plugin implements :func:`startTest`, :func:`testOutcome` and
:func:`stopTestRun` to compile and then output a test report in
junit-xml format. By default, the report is written to a file called
``nose2-junit.xml`` in the current working directory. 

You can configure the output filename by setting ``path`` in a ``[junit-xml]``
section in a config file.  Unicode characters which are invalid in XML 1.0
are replaced with the ``U+FFFD`` replacement character.  In the case that your
software throws an error with an invalid byte string.  

By default, the ranges of discouraged characters are replaced as well.  This can be
changed by setting the ``keep_restricted`` configuration variable to ``True``.

"""
# Based on unittest2/plugins/junitxml.py,
# which is itself based on the junitxml plugin from py.test
import os.path
import time
import re
import sys
import json
from xml.etree import ElementTree as ET

import six

from nose2 import events, result, util

__unittest = True


class JUnitXmlReporter(events.Plugin):
    """Output junit-xml test report to file"""
    configSection = 'junit-xml'
    commandLineSwitch = ('X', 'junit-xml', 'Generate junit-xml output report')

    def __init__(self):
        self.path = os.path.realpath(
            self.config.as_str('path', default='nose2-junit.xml'))
        self.keep_restricted = self.config.as_bool('keep_restricted',
                                                   default=False)
        self.test_properties = self.config.as_str('test_properties',
                                                  default=None)
        if self.test_properties is not None:
            self.test_properties_path = os.path.realpath(self.test_properties)
        self.errors = 0
        self.failed = 0
        self.skipped = 0
        self.numtests = 0
        self.tree = ET.Element('testsuite')
        self._start = None

    def startTest(self, event):
        """Count test, record start time"""
        self.numtests += 1
        self._start = event.startTime

    def testOutcome(self, event):
        """Add test outcome to xml tree"""
        test = event.test
        testid = test.id().split('\n')[0]
        # split into module, class, method parts... somehow
        parts = testid.split('.')
        classname = '.'.join(parts[:-1])
        method = parts[-1]

        testcase = ET.SubElement(self.tree, 'testcase')
        testcase.set('time', "%.6f" % self._time())
        testcase.set('classname', classname)
        testcase.set('name', method)

        msg = ''
        if event.exc_info:
            msg = util.exc_info_to_string(event.exc_info, test)
        elif event.reason:
            msg = event.reason

        msg = string_cleanup(msg, self.keep_restricted)

        if event.outcome == result.ERROR:
            self.errors += 1
            error = ET.SubElement(testcase, 'error')
            error.set('message', 'test failure')
            error.text = msg
        elif event.outcome == result.FAIL and not event.expected:
            self.failed += 1
            failure = ET.SubElement(testcase, 'failure')
            failure.set('message', 'test failure')
            failure.text = msg
        elif event.outcome == result.PASS and not event.expected:
            self.skipped += 1
            skipped = ET.SubElement(testcase, 'skipped')
            skipped.set('message', 'test passes unexpectedly')
        elif event.outcome == result.SKIP:
            self.skipped += 1
            skipped = ET.SubElement(testcase, 'skipped')
        elif event.outcome == result.FAIL and event.expected:
            self.skipped += 1
            skipped = ET.SubElement(testcase, 'skipped')
            skipped.set('message', 'expected test failure')
            skipped.text = msg

        system_err = ET.SubElement(testcase, 'system-err')
        system_err.text = string_cleanup(
            '\n'.join(event.metadata.get('logs', '')),
            self.keep_restricted
        )

    def _check(self):
        if not os.path.exists(os.path.dirname(self.path)):
            raise IOError(2, 'JUnitXML: Parent folder does not exist for file',
                          self.path)
        if self.test_properties is not None:
            if not os.path.exists(self.test_properties_path):
                raise IOError(2, 'JUnitXML: Properties file does not exist',
                              self.test_properties_path)

    def stopTestRun(self, event):
        """Output xml tree to file"""
        self.tree.set('name', 'nose2-junit')
        self.tree.set('errors', str(self.errors))
        self.tree.set('failures', str(self.failed))
        self.tree.set('skipped', str(self.skipped))
        self.tree.set('tests', str(self.numtests))
        self.tree.set('time', "%.3f" % event.timeTaken)

        self._check()
        self._include_test_properties()
        self._indent_tree(self.tree)

        output = ET.ElementTree(self.tree)
        output.write(self.path, encoding="utf-8")

    def _include_test_properties(self):
        """Include test properties in xml tree"""
        if self.test_properties is None:
            return

        props = {}
        with open(self.test_properties_path) as data:
            try:
                props = json.loads(data.read())
            except ValueError:
                raise ValueError('JUnitXML: could not decode file: \'%s\'' %
                                 self.test_properties_path)

        properties = ET.SubElement(self.tree, 'properties')
        for key, val in props.items():
            prop = ET.SubElement(properties, 'property')
            prop.set('name', key)
            prop.set('value', val)

    def _indent_tree(self, elem, level=0):
        """In-place pretty formatting of the ElementTree structure."""
        i = "\n" + level * "  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self._indent_tree(elem, level + 1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    def _time(self):
        try:
            return time.time() - self._start
        except Exception:
            pass
        finally:
            self._start = None
        return 0


#
# xml utility functions
#

# six doesn't include a unichr function


def _unichr(string):
    if six.PY3:
        return chr(string)
    else:
        return unichr(string)

# etree outputs XML 1.0 so the 1.1 Restricted characters are invalid.
# and there are no characters that can be given as entities aside
# form & < > ' " which ever have to be escaped (etree handles these fine)
ILLEGAL_RANGES = [(0x00, 0x08), (0x0B, 0x0C), (0x0E, 0x1F),
                  (0xD800, 0xDFFF), (0xFFFE, 0xFFFF)]
# 0xD800 thru 0xDFFF are technically invalid in UTF-8 but PY2 will encode
# bytes into these but PY3 will do a replacement

# Other non-characters which are not strictly forbidden but
# discouraged.
RESTRICTED_RANGES = [(0x7F, 0x84), (0x86, 0x9F), (0xFDD0, 0xFDDF)]
# check for a wide build
if sys.maxunicode > 0xFFFF:
    RESTRICTED_RANGES += [(0x1FFFE, 0x1FFFF), (0x2FFFE, 0x2FFFF),
                          (0x3FFFE, 0x3FFFF), (0x4FFFE, 0x4FFFF),
                          (0x5FFFE, 0x5FFFF), (0x6FFFE, 0x6FFFF),
                          (0x7FFFE, 0x7FFFF), (0x8FFFE, 0x8FFFF),
                          (0x9FFFE, 0x9FFFF), (0xAFFFE, 0xAFFFF),
                          (0xBFFFE, 0xBFFFF), (0xCFFFE, 0xCFFFF),
                          (0xDFFFE, 0xDFFFF), (0xEFFFE, 0xEFFFF),
                          (0xFFFFE, 0xFFFFF), (0x10FFFE, 0x10FFFF)]

ILLEGAL_REGEX_STR = \
    six.u('[') + \
    six.u('').join(["%s-%s" % (_unichr(l), _unichr(h))
                    for (l, h) in ILLEGAL_RANGES]) + \
    six.u(']')
RESTRICTED_REGEX_STR = \
    six.u('[') + \
    six.u('').join(["%s-%s" % (_unichr(l), _unichr(h))
                    for (l, h) in RESTRICTED_RANGES]) + \
    six.u(']')

_ILLEGAL_REGEX = re.compile(ILLEGAL_REGEX_STR, re.U)
_RESTRICTED_REGEX = re.compile(RESTRICTED_REGEX_STR, re.U)


def string_cleanup(string, keep_restricted=False):
    if not issubclass(type(string), six.text_type):
        string = six.text_type(string, encoding='utf-8', errors='replace')

    string = _ILLEGAL_REGEX.sub(six.u('\uFFFD'), string)
    if not keep_restricted:
        string = _RESTRICTED_REGEX.sub(six.u('\uFFFD'), string)

    return string



"""
Loader that implements the ``load_tests`` protocol.

This plugin implements the ``load_tests`` protocol as detailed in the
documentation for :mod:`unittest2`.

See the `load_tests protocol`_ documentation for more information.

.. warning ::

   Test suites using the ``load_tests`` protocol do not work correctly
   with the :mod:`multiprocess` plugin as of nose2 04. This will be
   fixed in a future release.

.. _load_tests protocol: http://docs.python.org/library/unittest.html#load-tests-protocol
"""
from fnmatch import fnmatch
import logging

from nose2 import events, util


log = logging.getLogger(__name__)


class LoadTestsLoader(events.Plugin):

    """Loader plugin that implements load_tests."""
    alwaysOn = True
    configSection = 'load_tests'
    _loading = False

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)

    def moduleLoadedSuite(self, event):
        """Run ``load_tests`` in a module.

        May add to or filter tests loaded in module.

        """
        module = event.module
        load_tests = getattr(module, 'load_tests', None)
        if not load_tests:
            return
        try:
            event.suite = load_tests(
                event.loader, event.suite, self.session.testFilePattern)
        except Exception as exc:
            log.exception(
                "Failed to load tests from %s via load_tests", module)
            suite = event.loader.suiteClass()
            suite.addTest(event.loader.failedLoadTests(module.__name__, exc))
            event.handled = True
            return suite

    def handleDir(self, event):
        """Run ``load_tests`` in packages.

        If a package itself matches the test file pattern, run
        ``load_tests`` in its :file:`__init__.py`, and stop default test
        discovery for that package.

        """
        if self._loading:
            return

        if (self._match(event.name, event.pattern) and
            util.ispackage(event.path)):
            name, _package_path = util.name_from_path(event.path)
            module = util.module_from_name(name)

            load_tests = getattr(module, 'load_tests', None)
            if not load_tests:
                return
            self._loading = True
            try:
                suite = event.loader.suiteClass()
                try:
                    suite = load_tests(event.loader, suite, event.pattern)
                except Exception as exc:
                    log.exception(
                        "Failed to load tests from %s via load_tests", module)
                    suite.addTest(
                        event.loader.failedLoadTests(module.__name__, exc))

                event.handled = True
                return suite
            finally:
                self._loading = False

    def _match(self, filename, pattern):
        return fnmatch(filename, pattern)



from nose2 import events, loader, session
from nose2.tests._common import TestCase


class TestPluggableTestLoader(TestCase):

    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(self.session)

    def test_failed_load_tests_exception(self):
        suite = self.loader.failedLoadTests('test', RuntimeError('err'))
        tc = suite._tests[0]
        with self.assertRaises(RuntimeError) as cm:
            tc.test()
        self.assertEqual(cm.exception.args, ('err', ))

    def test_failed_load_tests_exc_info(self):
        suite = self.loader.failedLoadTests(
            'test', (RuntimeError, RuntimeError('err'), None))
        tc = suite._tests[0]
        with self.assertRaises(RuntimeError) as cm:
            tc.test()
        self.assertEqual(cm.exception.args, ('err', ))

    def test_load_from_module_calls_hook(self):
        self.session.hooks.register('loadTestsFromModule', FakePlugin())
        evt = events.LoadFromModuleEvent(self.loader, 'some_module')
        self.session.hooks.loadTestsFromModule(evt)
        self.assertTrue(evt.fakeLoadFromModule,
                        "FakePlugin.loadTestsFromModule() was not called")

    def test_load_from_name_calls_hook(self):
        self.session.hooks.register('loadTestsFromName', FakePlugin())
        evt = events.LoadFromNameEvent(self.loader,
                                       'some_name',
                                       'some_module')
        self.session.hooks.loadTestsFromName(evt)
        self.assertTrue(evt.fakeLoadFromName,
                        "FakePlugin.fakeLoadFromName() was not called")

    def test_load_from_names_calls_hook(self):
        self.session.hooks.register('loadTestsFromNames', FakePlugin())
        evt = events.LoadFromNamesEvent(self.loader,
                                        ['some_name'],
                                        'some_module')
        self.session.hooks.loadTestsFromNames(evt)
        self.assertTrue(evt.fakeLoadFromNames,
                        "FakePlugin.fakeLoadFromNames() was not called")

    def test_loader_from_names_calls_module_hook(self):
        fake_plugin = FakePlugin()
        self.session.hooks.register('loadTestsFromModule', fake_plugin)
        self.loader.loadTestsFromNames([], 'some_module')
        self.assertTrue(fake_plugin.fakeLoadFromModule,
                        "FakePlugin.loadTestsFromModule() was not called")

    def test_loader_from_names_calls_name_hook(self):
        fake_plugin = FakePlugin()
        self.session.hooks.register('loadTestsFromName', fake_plugin)
        self.loader.loadTestsFromNames(['some_name'])
        self.assertTrue(fake_plugin.fakeLoadFromName,
                        "FakePlugin.loadTestsFromName() was not called")

    def test_loader_from_names_calls_names_hook(self):
        fake_plugin = FakePlugin()
        self.session.hooks.register('loadTestsFromNames', fake_plugin)
        self.loader.loadTestsFromNames(['some_name'])
        self.assertTrue(fake_plugin.fakeLoadFromNames,
                        "FakePlugin.loadTestsFromNames() was not called")


class FakePlugin(object):

    def __init__(self):
        self.fakeLoadFromModule = False
        self.fakeLoadFromName = False
        self.fakeLoadFromNames = False

    def loadTestsFromModule(self, event):
        event.fakeLoadFromModule = True
        self.fakeLoadFromModule = True

    def loadTestsFromName(self, event):
        event.fakeLoadFromName = True
        self.fakeLoadFromName = True

    def loadTestsFromNames(self, event):
        event.fakeLoadFromNames = True
        self.fakeLoadFromNames = True


import unittest


class TestCase(unittest.TestCase):

    def test_a(self):
        pass

    def test_b(self):
        pass

    def test_c(self):
        pass


def load_tests(loader, tests, pattern):
    class InnerTest(unittest.TestCase):

        def test_d(self):
            pass
    tests.addTest(InnerTest('test_d'))
    return tests



import logging
import unittest

from pkg2 import get_one


log = logging.getLogger(__name__)
log.debug("module imported")


def test():
    log.debug("test run")
    assert get_one() == 1


def test_fail():
    log.debug("test_fail run")
    assert get_one() == 2


class Tests(unittest.TestCase):

    def test_fail2(self):
        log.debug("test_fail2 run")
        self.assertEqual(get_one(), 4)



import unittest

class Layer(object):

    @classmethod
    def setUp(cls):
        pass

    @classmethod
    def testTearDown(cls):
        raise RuntimeError('Bad Error in Layer testTearDown!')

class Test(unittest.TestCase):
    layer = Layer
    
    def testPass(self):
        pass



class Test(object):

    @classmethod
    def setUpClass(cls):
        cls.setup = 1

    @classmethod
    def tearDownClass(cls):
        del cls.setup

    def setUp(self):
        self.test_setup = 1

    def tearDown(self):
        del self.test_setup

    def test(self):
        assert self.test_setup
        assert self.setup

    def test_gen(self):
        def check(a):
            assert self.test_setup
            assert self.setup
        for i in range(0, 2):
            yield check, i

    def test_params(self, a):
        assert self.test_setup
        assert self.setup
    test_params.paramList = (1, 2)



from nose2.tests._common import FunctionalTestCase


class TestLayers(FunctionalTestCase):

    def test_runs_layer_fixtures(self):
        proc = self.runIn(
            'scenario/layers',
            '-v',
            '--plugin=nose2.plugins.layers')
        self.assertTestRunOutputMatches(proc, stderr='Ran 8 tests')
        self.assertEqual(proc.poll(), 0)

    def test_scenario_fails_without_plugin(self):
        proc = self.runIn(
            'scenario/layers',
            '-v')
        self.assertTestRunOutputMatches(proc, stderr='Ran 8 tests')
        self.assertTestRunOutputMatches(proc, stderr=r'FAILED \(failures=7\)')
        self.assertEqual(proc.poll(), 1)

    def test_methods_run_once_per_class(self):
        proc = self.runIn(
            'scenario/layers_with_inheritance',
            '-v',
            '--plugin=nose2.plugins.layers')

        expected = ('^'
                    'L1 setUp\n'
                    'L2 setUp\n'

                    'L1 testSetUp\n'
                    'L2 testSetUp\n'
                    'Run test1\n'
                    'L2 testTearDown\n'
                    'L1 testTearDown\n'

                    'L1 testSetUp\n'
                    'L2 testSetUp\n'
                    'Run test2\n'
                    'L2 testTearDown\n'
                    'L1 testTearDown\n'

                    'L1 tearDown\n'
                    '$')
        self.assertTestRunOutputMatches(proc, stdout=expected)
        self.assertEqual(proc.poll(), 0)

    def test_layer_reporter_output(self):
        proc = self.runIn(
            'scenario/layers',
            '-v',
            '--plugin=nose2.plugins.layers',
            '--layer-reporter')
        expect = r"""test \(test_layers.NoLayer\) ... ok
Base
  test \(test_layers.Outer\) ... ok
  LayerD
    test \(test_layers.InnerD\) ... ok
  LayerA
    test \(test_layers.InnerA\) ... ok
  LayerB
    LayerB_1
      test \(test_layers.InnerB_1\) ... ok
    LayerC
      test \(test_layers.InnerC\) ... ok
      test2 \(test_layers.InnerC\) ... ok
    LayerA_1
      test \(test_layers.InnerA_1\) ... ok""".split("\n")
        self.assertTestRunOutputMatches(proc, stderr='Ran 8 tests')
        for line in expect:
            self.assertTestRunOutputMatches(proc, stderr=line)
        self.assertEqual(proc.poll(), 0)

    def test_layer_reporter_error_output(self):
        proc = self.runIn(
            'scenario/layers_with_errors',
            '--plugin=nose2.plugins.layers',
            '--layer-reporter')
        expect = [
            r'ERROR: fixture with a value test_err '
            '\(test_layers_with_errors.Test\)',
            'ERROR: A test scenario with errors should check for an attribute '
            'that does not exist and raise an error',
            r'FAIL: fixture with a value test_fail '
            '\(test_layers_with_errors.Test\)',
            'FAIL: A test scenario with errors should check that value == 2 '
            'and fail']
        for line in expect:
            self.assertTestRunOutputMatches(proc, stderr=line)
        self.assertEqual(proc.poll(), 1)

    def test_layers_and_attributes(self):
        proc = self.runIn(
            'scenario/layers_and_attributes',
            '-v',
            '--plugin=nose2.plugins.attrib',
            '--plugin=nose2.plugins.layers',
            '-A',
            'a=1')
        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test')
        self.assertEqual(proc.poll(), 0)

    def test_teardown_fail(self):
        proc = self.runIn('scenario/layers_with_errors',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'test_layer_teardown_fail')
        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test in')
        self.assertTestRunOutputMatches(proc, stderr='ERROR: LayerSuite')
        self.assertTestRunOutputMatches(proc, stderr='FAIL')
        self.assertTestRunOutputMatches(proc, stderr='Bad Error in Layer testTearDown')

    def test_setup_fail(self):
        proc = self.runIn('scenario/layers_with_errors',
                          '--plugin=nose2.plugins.layers',
                          '-v',
                          'test_layer_setup_fail')
        self.assertTestRunOutputMatches(proc, stderr='Ran 0 tests in')
        self.assertTestRunOutputMatches(proc, stderr='ERROR: LayerSuite')
        self.assertTestRunOutputMatches(proc, stderr='FAIL')
        self.assertTestRunOutputMatches(proc, stderr='Bad Error in Layer setUp!')



"""
This plugin implements :func:`startTestRun`, setting a test executor
(``event.executeTests``) that just collects tests without executing
them. To do so it calls result.startTest, result.addSuccess and
result.stopTest for ech test, without calling the test itself.
"""
from nose2.events import Plugin
from nose2.compat import unittest


__unittest = True


class CollectOnly(Plugin):

    """Collect but don't run tests"""

    configSection = 'collect-only'
    commandLineSwitch = (None, 'collect-only',
                         'Collect and output test names; do not run any tests')
    _mpmode = False

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)
        self._mpmode = True

    def startTestRun(self, event):
        """Replace ``event.executeTests``"""
        if self._mpmode:
            return
        event.executeTests = self.collectTests

    def startSubprocess(self, event):
        event.executeTests = self.collectTests

    def collectTests(self, suite, result):
        """Collect tests, but don't run them"""
        for test in suite:
            if isinstance(test, unittest.BaseTestSuite):
                self.collectTests(test, result)
                continue
            result.startTest(test)
            result.addSuccess(test)
            result.stopTest(test)




import nose2
import unittest
import logging

log = logging.getLogger(__name__)

class Test(unittest.TestCase):

    def test_logging_keeps_copies_of_mutable_objects(self):
        d = {}
        log.debug("foo: %s", d)
        d["bar"] = "baz"
        self.assert_(False)

if __name__ == '__main__':
    nose2.main()